@@ -1,6 +1,6 @@
 /*******************************************************************************
  * shroudBNC - an object-oriented framework for IRC                            *
- * Copyright (C) 2005-2007 Gunnar Beutner                                      *
+ * Copyright (C) 2005-2014 Gunnar Beutner                                      *
  *                                                                             *
  * This program is free software; you can redistribute it and/or               *
  * modify it under the terms of the GNU General Public License                 *
@@ -54,7 +54,7 @@ void die(void) {
 void setctx(const char *ctx) {
 	char *Suffix;
 	CUser *User;
-	unsigned int i;
+	int i;
 	time_t TS;
 	char *CtxDup;
 
@@ -104,23 +104,19 @@ void setctx(const char *ctx) {
 
 const char *getctx(int ts) {
 	static char *Context = NULL;
-	unsigned int i;
+	int i, rc;
 	CUser *User;
 	time_t TS;
 
-	g_free(Context);
+	gfree(Context);
 
 	if (g_CurrentClient != NULL && ts) {
 		User = g_CurrentClient->GetOwner();
 
 		if (User == NULL) {
-			g_asprintf(&Context, "");
-
-			return Context;
-		}
-
-		if (g_CurrentClient == User->GetClientConnectionMultiplexer()) {
-			g_asprintf(&Context, "%s<*", g_Context);
+			rc = asprintf(&Context, "");
+		} else 	if (g_CurrentClient == User->GetClientConnectionMultiplexer()) {
+			rc = asprintf(&Context, "%s<*", g_Context);
 		} else {
 			TS = 0;
 
@@ -132,10 +128,14 @@ const char *getctx(int ts) {
 				}
 			}
 
-			g_asprintf(&Context, "%s<%d", g_Context, TS);
+			rc = asprintf(&Context, "%s<%d", g_Context, (int)TS);
 		}
 	} else {
-		g_asprintf(&Context, "%s", g_Context);
+		rc = asprintf(&Context, "%s", g_Context);
+	}
+
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
 	}
 
 	return Context;
@@ -148,7 +148,7 @@ const char *bncuserlist(void) {
 	int argc = 0;
 	const char** argv = (const char**)malloc(Count * sizeof(const char*));
 
-	CHashtable<CUser *, false, 512> *Users = g_Bouncer->GetUsers();
+	CHashtable<CUser *, false> *Users = g_Bouncer->GetUsers();
 
 	i = 0;
 	while (hash_t<CUser *> *User = Users->Iterate(i++)) {
@@ -179,12 +179,12 @@ const char* internalchannels(void) {
 	if (!IRC)
 		throw "User is not connected to an IRC server.";
 
-	CHashtable<CChannel*, false, 16>* H = IRC->GetChannels();
+	CHashtable<CChannel*, false>* H = IRC->GetChannels();
 
 	if (H == NULL)
 		return NULL;
 
-	int Count = Count = H->GetLength();
+	int Count = H->GetLength();
 
 	const char** argv = (const char**)malloc(Count * sizeof(const char*));
 
@@ -469,7 +469,7 @@ const char* internalbinds(void) {
 	return Out;
 }
 
-int putserv(const char* text) {
+int putserv(const char* text, const char *option) {
 	CUser* Context = g_Bouncer->GetUser(g_Context);
 
 	if (Context == NULL)
@@ -480,7 +480,11 @@ int putserv(const char* text) {
 	if (!IRC)
 		return 0;
 
-	IRC->WriteLine("%s", text);
+	if (option != NULL && strcasecmp(option, "-next") == 0) {
+		IRC->GetQueueMiddle()->QueueItemNext(text);
+	} else {
+		IRC->GetQueueMiddle()->QueueItem(text);
+	}
 
 	return 1;
 }
@@ -628,7 +632,7 @@ const char* internalchanlist(const char* Channel) {
 	if (!Chan)
 		return NULL;
 
-	const CHashtable<CNick*, false, 64>* Names = Chan->GetNames();
+	const CHashtable<CNick*, false>* Names = Chan->GetNames();
 
 	int Count = Names->GetLength();
 	const char** argv = (const char**)malloc(Count * sizeof(const char*));
@@ -794,7 +798,7 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 	static char *Buffer = NULL;
 
 	if (Buffer != NULL) {
-		g_free(Buffer);
+		gfree(Buffer);
 		Buffer = NULL;
 	}
 
@@ -815,7 +819,11 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 		else
 			return NULL;
 	} else if (strcasecmp(Type, "port") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetPort());
+		int rc = asprintf(&Buffer, "%d", Context->GetPort());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		return Buffer;
 	} else if (strcasecmp(Type, "realname") == 0)
@@ -833,7 +841,11 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 	else if (strcasecmp(Type, "channels") == 0)
 		return Context->GetConfigChannels();
 	else if (strcasecmp(Type, "uptime") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetIRCUptime());
+		int rc = asprintf(&Buffer, "%d", Context->GetIRCUptime());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		return Buffer;
 	} else if (strcasecmp(Type, "lock") == 0)
@@ -861,7 +873,11 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 		else
 			return Client->GetPeerName();
 	} else if (strcasecmp(Type, "clientcount") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetClientConnections()->GetLength());
+		int rc = asprintf(&Buffer, "%d", Context->GetClientConnections()->GetLength());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		return Buffer;
 	} else if (strcasecmp(Type, "tag") == 0) {
@@ -894,23 +910,27 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 
 		return List;
 	} else if (strcasecmp(Type, "seen") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetLastSeen());
+		int rc = asprintf(&Buffer, "%d", (int)Context->GetLastSeen());
 
-		return Buffer;
-	} else if (strcasecmp(Type, "appendts") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetConfig()->ReadInteger("user.ts") != 0 ? 1 : 0);
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		return Buffer;
 	} else if (strcasecmp(Type, "quitasaway") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetConfig()->ReadInteger("user.quitaway") != 0 ? 1 : 0);
+		int rc = asprintf(&Buffer, "%d", Context->GetUseQuitReason() ? 1 : 0);
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		return Buffer;
 	} else if (strcasecmp(Type, "automodes") == 0) {
-		return Context->GetConfig()->ReadString("user.automodes");
+		return Context->GetAutoModes();
 	} else if (strcasecmp(Type, "dropmodes") == 0) {
-		return Context->GetConfig()->ReadString("user.dropmodes");
+		return Context->GetDropModes();
 	} else if (strcasecmp(Type, "suspendreason") == 0) {
-		return Context->GetConfig()->ReadString("user.suspend");
+		return Context->GetSuspendReason();
 	} else if (strcasecmp(Type, "ssl") == 0) {
 		return Context->GetSSL() ? "1" : "0";
 	} else if (strcasecmp(Type, "sslclient") == 0) {
@@ -919,41 +939,30 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 		if (!Client)
 			return NULL;
 		else {
-			g_asprintf(&Buffer, "%d", Client->IsSSL() ? 1 : 0);
+			int rc = asprintf(&Buffer, "%d", Client->IsSSL() ? 1 : 0);
+
+			if (RcFailed(rc)) {
+				g_Bouncer->Fatal();
+			}
 
 			return Buffer;
 		}
-	} else if (strcasecmp(Type, "ipv6") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetIPv6() ? 1 : 0);
-
-		return Buffer;
 	} else if (strcasecmp(Type, "ident") == 0) {
 		return Context->GetIdent();
-	} else if (strcasecmp(Type, "timezone") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetGmtOffset());
-
-		return Buffer;
 	} else if (strcasecmp(Type, "localip") == 0) {
-		const utility_t *Utilities;
 		IRC = Context->GetIRCConnection();
 
-		Utilities = g_Bouncer->GetUtilities();
-
 		if (IRC != NULL && IRC->GetLocalAddress() != NULL) {
-			return Utilities->IpToString(IRC->GetLocalAddress());
+			return IpToString(IRC->GetLocalAddress());
 		} else {
 			return NULL;
 		}
 	} else if (strcasecmp(Type, "lean") == 0) {
-		g_asprintf(&Buffer, "%d", Context->GetLeanMode());
+		int rc = asprintf(&Buffer, "%d", Context->GetLeanMode());
 
-		return Buffer;
-	} else if (strcasecmp(Type, "memory") == 0) {
-		g_asprintf(&Buffer, "%d", Context->MemoryGetSize());
-
-		return Buffer;
-	} else if (strcasecmp(Type, "memorylimit") == 0) {
-		g_asprintf(&Buffer, "%d", Context->MemoryGetLimit());
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		return Buffer;
 	} else if (strcasecmp(Type, "channelsort") == 0) {
@@ -965,8 +974,12 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 
 		char** argv = (char**)malloc(Count * sizeof(const char*));
 
-		for (unsigned int i = 0; i < Count; i++) {
-			g_asprintf(&Item, "%s<%d", Context->GetUsername(), (*Clients)[i].Creation);
+		for (int i = 0; i < Count; i++) {
+			int rc = asprintf(&Item, "%s<%d", Context->GetUsername(), (int)(*Clients)[i].Creation);
+
+			if (RcFailed(rc)) {
+				g_Bouncer->Fatal();
+			}
 
 			argv[i] = Item;
 		}
@@ -979,15 +992,23 @@ const char* getbncuser(const char* User, const char* Type, const char* Parameter
 
 		List = Tcl_Merge(Count, const_cast<char **>(argv));
 
-		for (unsigned int i = 0; i < Count; i++) {
-			g_free(argv[i]);
+		for (int i = 0; i < Count; i++) {
+			free(argv[i]);
 		}
 
 		free(argv);
 
 		return List;
+	} else if (strcasecmp(Type, "autobacklog") == 0) {
+		return Context->GetAutoBacklog();
+	} else if (strcasecmp(Type, "sysnotices") == 0) {
+		return Context->GetSystemNotices() ? "1" : "0";
 	} else {
-		throw "Type should be one of: server port serverpass client clientcount realname nick awaynick away awaymessage uptime lock admin hasserver hasclient vhost channels tag delayjoin seen appendts quitasaway automodes dropmodes suspendreason ssl sslclient realserver ident tags ipv6 timezone localip lean memory memorylimit channelsort sessions";
+		throw "Type should be one of: server port serverpass client clientcount "
+			"realname nick awaynick away awaymessage uptime lock admin hasserver "
+			"hasclient vhost channels tag delayjoin seen quitasaway automodes "
+			"dropmodes suspendreason ssl sslclient realserver ident tags localip "
+			"lean memory memorylimit channelsort sessions autobacklog sysnotices";
 	}
 }
 
@@ -1030,30 +1051,31 @@ int setbncuser(const char* User, const char* Type, const char* Value, const char
 			Context->Unlock();
 	} else if (strcasecmp(Type, "admin") == 0)
 		Context->SetAdmin(Value ? (atoi(Value) ? true : false) : false);
-	else if (strcasecmp(Type, "tag") == 0 && Value) {
+	else if (strcasecmp(Type, "tag") == 0 && Value)
 		Context->SetTagString(Value, Parameter2);
-	} else if (strcasecmp(Type, "appendts") == 0)
-		Context->GetConfig()->WriteString("user.ts", Value);
 	else if (strcasecmp(Type, "quitasaway") == 0)
-		Context->GetConfig()->WriteString("user.quitaway", Value);
+		Context->SetUseQuitReason(Value ? (atoi(Value) ? true : false) : false);
 	else if (strcasecmp(Type, "automodes") == 0)
-		Context->GetConfig()->WriteString("user.automodes", Value);
+		Context->SetAutoModes(Value);
 	else if (strcasecmp(Type, "dropmodes") == 0)
-		Context->GetConfig()->WriteString("user.dropmodes", Value);
+		Context->SetDropModes(Value);
 	else if (strcasecmp(Type, "suspendreason") == 0)
-		Context->GetConfig()->WriteString("user.suspend", Value);
-	else if (strcasecmp(Type, "ipv6") == 0)
-		Context->SetIPv6(Value ? (atoi(Value) ? true : false) : false);
+		Context->SetSuspendReason(Value);
 	else if (strcasecmp(Type, "ident") == 0)
 		Context->SetIdent(Value);
-	else if (strcasecmp(Type, "timezone") == 0)
-		Context->SetGmtOffset(atoi(Value));
-	else if (strcmp(Type, "lean") == 0)
+	else if (strcasecmp(Type, "lean") == 0)
 		Context->SetLeanMode(atoi(Value));
-	else if (strcmp(Type, "channelsort") == 0)
+	else if (strcasecmp(Type, "channelsort") == 0)
 		Context->SetChannelSortMode(Value);
+	else if (strcasecmp(Type, "autobacklog") == 0)
+		Context->SetAutoBacklog(Value);
+	else if (strcasecmp(Type, "sysnotices") == 0)
+		Context->SetSystemNotices(Value);
 	else
-		throw "Type should be one of: server port serverpass realname nick awaynick away awaymessage lock admin channels tag vhost delayjoin password appendts quitasaway automodes dropmodes suspendreason ident ipv6 timezone lean channelsort";
+		throw "Type should be one of: server port serverpass realname nick awaynick "
+			"away awaymessage lock admin channels tag vhost delayjoin password "
+			"quitasaway automodes dropmodes suspendreason ident lean channelsort "
+			"autobacklog sysnotices";
 
 	return 1;
 }
@@ -1221,6 +1243,11 @@ int internalgetchanidle(const char* Nick, const char* Channel) {
 }
 
 int ticklerand(int limit) {
+	if (limit == 0) {
+		/* not entirely correct, but certainly better than crashing */
+		return 0;
+	}
+
 	return rand() % limit;
 }
 
@@ -1252,7 +1279,7 @@ int floodcontrol(const char* Function) {
 	int Result;
 
 	if (strcasecmp(Function, "bytes") == 0)
-		Result = FloodControl->GetBytes();
+		Result = 0; /* not supported anymore */
 	else if (strcasecmp(Function, "items") == 0)
 		Result = FloodControl->GetQueueSize();
 	else if (strcasecmp(Function, "on") == 0) {
@@ -1335,7 +1362,7 @@ int queuesize(const char* Queue) {
 	return Size;
 }
 
-int puthelp(const char* text) {
+int puthelp(const char* text, const char *option) {
 	CUser* Context = g_Bouncer->GetUser(g_Context);
 
 	if (Context == NULL)
@@ -1346,12 +1373,16 @@ int puthelp(const char* text) {
 	if (!IRC)
 		return 0;
 
-	IRC->GetQueueLow()->QueueItem(text);
+	if (option != NULL && strcasecmp(option, "-next") == 0) {
+		IRC->GetQueueLow()->QueueItemNext(text);
+	} else {
+		IRC->GetQueueLow()->QueueItem(text);
+	}
 
 	return 1;
 }
 
-int putquick(const char* text) {
+int putquick(const char* text, const char *option) {
 	CUser* Context = g_Bouncer->GetUser(g_Context);
 
 	if (Context == NULL)
@@ -1362,7 +1393,11 @@ int putquick(const char* text) {
 	if (!IRC)
 		return 0;
 
-	IRC->GetQueueHigh()->QueueItem(text);
+	if (option != NULL && strcasecmp(option, "-next") == 0) {
+		IRC->GetQueueHigh()->QueueItemNext(text);
+	} else {
+		IRC->GetQueueHigh()->QueueItem(text);
+	}
 
 	return 1;
 }
@@ -1434,20 +1469,35 @@ const char* bncmodules(void) {
 	int a = 0;
 	char** List = (char**)malloc(Modules->GetLength() * sizeof(const char*));
 
-	for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+	for (int i = 0; i < Modules->GetLength(); i++) {
 		char *BufId, *Buf1, *Buf2;
+		int rc;
 
-		g_asprintf(&BufId, "%d", i);
-		g_asprintf(&Buf1, "%p", Modules->Get(i)->GetHandle());
-		g_asprintf(&Buf2, "%p", Modules->Get(i)->GetModule());
+		rc = asprintf(&BufId, "%d", i);
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
+
+		rc = asprintf(&Buf1, "%p", Modules->Get(i)->GetHandle());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
+
+		rc = asprintf(&Buf2, "%p", Modules->Get(i)->GetModule());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		const char* Mod[4] = { BufId, Modules->Get(i)->GetFilename(), Buf1, Buf2 };
 
 		List[a++] = Tcl_Merge(4, const_cast<char **>(Mod));
 
-		g_free(BufId);
-		g_free(Buf1);
-		g_free(Buf2);
+		free(BufId);
+		free(Buf1);
+		free(Buf2);
 	}
 
 	static char* Mods = NULL;
@@ -1523,7 +1573,7 @@ void haltoutput(void) {
 const char* bnccommand(const char* Cmd, const char* Parameters) {
 	const CVector<CModule *> *Modules = g_Bouncer->GetModules();
 
-	for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+	for (int i = 0; i < Modules->GetLength(); i++) {
 		const char* Result = Modules->Get(i)->Command(Cmd, Parameters);
 
 		if (Result)
@@ -1544,7 +1594,7 @@ void debugout(const char* String) {
 #ifdef _WIN32
 	OutputDebugString(String);
 	OutputDebugString("\n");
-#endif
+#endif /* _WIN32 */
 }
 
 void putlog(const char *Text) {
@@ -1605,7 +1655,8 @@ void bncjoinchans(const char* User) {
 		Context->GetIRCConnection()->JoinChannels();
 }
 
-int internallisten(unsigned short Port, const char* Type, const char* Options, const char* Flag, bool SSL, const char *BindIp) {
+int internallisten(unsigned short Port, const char* Type, const char* Options,
+		const char* Flag, bool SSL, const char *BindIp) {
 	if (strcasecmp(Type, "script") == 0) {
 		if (Options == NULL)
 			throw "You need to specifiy a control proc.";
@@ -1631,7 +1682,7 @@ int internallisten(unsigned short Port, const char* Type, const char* Options, c
 		while ((Socket = g_Bouncer->GetSocketByClass("CTclSocket", i++)) != NULL) {
 			sockaddr_in saddr;
 			socklen_t saddrSize = sizeof(saddr);
-			safe_getsockname(Socket->PollFd->fd, (sockaddr*)&saddr, &saddrSize);
+			getsockname(Socket->PollFd->fd, (sockaddr*)&saddr, &saddrSize);
 
 			if (ntohs(saddr.sin_port) == Port) {
 				Socket->Events->Destroy();
@@ -1647,9 +1698,14 @@ int internallisten(unsigned short Port, const char* Type, const char* Options, c
 
 void control(int Socket, const char* Proc) {
 	char *Buf;
-	g_asprintf(&Buf, "%d", Socket);
+	int rc = asprintf(&Buf, "%d", Socket);
+
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
+	}
+
 	CTclClientSocket* SockPtr = g_TclClientSockets->Get(Buf);
-	g_free(Buf);
+	gfree(Buf);
 
 	if (!SockPtr || !g_Bouncer->IsRegisteredSocket(SockPtr))
 		throw "Invalid socket.";
@@ -1659,9 +1715,14 @@ void control(int Socket, const char* Proc) {
 
 int internalvalidsocket(int Socket) {
 	char *Buf;
-	g_asprintf(&Buf, "%d", Socket);
+	int rc = asprintf(&Buf, "%d", Socket);
+
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
+	}
+
 	CTclClientSocket* SockPtr = g_TclClientSockets->Get(Buf);
-	g_free(Buf);
+	gfree(Buf);
 
 	if (!SockPtr || !g_Bouncer->IsRegisteredSocket(SockPtr))
 		return false;
@@ -1671,9 +1732,14 @@ int internalvalidsocket(int Socket) {
 
 void internalsocketwriteln(int Socket, const char* Line) {
 	char *Buf;
-	g_asprintf(&Buf, "%d", Socket);
+	int rc = asprintf(&Buf, "%d", Socket);
+
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
+	}
+
 	CTclClientSocket* SockPtr = g_TclClientSockets->Get(Buf);
-	g_free(Buf);
+	gfree(Buf);
 
 	if (!SockPtr || !g_Bouncer->IsRegisteredSocket(SockPtr))
 		throw "Invalid socket pointer.";
@@ -1694,9 +1760,14 @@ int internalconnect(const char* Host, unsigned short Port, bool SSL) {
 
 const char *internalgetipforsocket(int Socket) {
 	char *Buf;
-	g_asprintf(&Buf, "%d", Socket);
+	int rc = asprintf(&Buf, "%d", Socket);
+
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
+	}
+
 	CTclClientSocket* SockPtr = g_TclClientSockets->Get(Buf);
-	g_free(Buf);
+	gfree(Buf);
 
 	if (!SockPtr || !g_Bouncer->IsRegisteredSocket(SockPtr))
 		throw "Invalid socket pointer.";
@@ -1706,19 +1777,20 @@ const char *internalgetipforsocket(int Socket) {
 	if (RemoteAddr == NULL) {
 		return NULL;
 	} else {
-		const utility_t *Utilities;
-
-		Utilities = g_Bouncer->GetUtilities();
-
-		return Utilities->IpToString(RemoteAddr);
+		return IpToString(RemoteAddr);
 	}
 }
 
 void internalclosesocket(int Socket) {
 	char *Buf;
-	g_asprintf(&Buf, "%d", Socket);
+	int rc = asprintf(&Buf, "%d", Socket);
+
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
+	}
+
 	CTclClientSocket* SockPtr = g_TclClientSockets->Get(Buf);
-	g_free(Buf);
+	gfree(Buf);
 
 	if (!SockPtr || !g_Bouncer->IsRegisteredSocket(SockPtr))
 		throw "Invalid socket pointer.";
@@ -1813,13 +1885,17 @@ char* chanbans(const char* Channel) {
 		char *Timestamp;
 		const ban_t *Ban = BanHash->Value;
 
-		g_asprintf(&Timestamp, "%d", Ban->Timestamp);
+		int rc = asprintf(&Timestamp, "%d", (int)Ban->Timestamp);
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
 
 		char* ThisBan[3] = { Ban->Mask, Ban->Nick, Timestamp };
 
 		char* List = Tcl_Merge(3, const_cast<char **>(ThisBan));
 
-		g_free(Timestamp);
+		gfree(Timestamp);
 
 		Blist = (char**)realloc(Blist, ++Bcount * sizeof(char*));
 
@@ -1886,7 +1962,7 @@ bool TclTimerProc(time_t Now, void* RawCookie) {
 
 		free(Cookie->proc);
 		free(Cookie->param);
-		free(Cookie);
+		delete Cookie;
 	}
 
 	return true;
@@ -1911,7 +1987,7 @@ int internaltimer(int Interval, bool Repeat, const char* Proc, const char* Param
 		n = &g_Timers[g_TimerCount - 1];
 	}
 	
-	*n = (tcltimer_t*)malloc(sizeof(tcltimer_t));
+	*n = new tcltimer_t;;
 
 	tcltimer_t* p = *n;
 
@@ -1932,11 +2008,12 @@ int internalkilltimer(const char* Proc, const char* Parameter) {
 		return 0;
 
 	for (int i = 0; i < g_TimerCount; i++) {
-		if (g_Timers[i] && strcmp(g_Timers[i]->proc, Proc) == 0 && (!Parameter || !g_Timers[i]->param || strcmp(Parameter, g_Timers[i]->param) == 0)) {
+		if (g_Timers[i] && strcmp(g_Timers[i]->proc, Proc) == 0 &&
+				(!Parameter || !g_Timers[i]->param || strcmp(Parameter, g_Timers[i]->param) == 0)) {
 			g_Timers[i]->timer->Destroy();
 			free(g_Timers[i]->proc);
 			free(g_Timers[i]->param);
-			free(g_Timers[i]);
+			delete g_Timers[i];
 
 			g_Timers[i] = NULL;
 
@@ -1954,21 +2031,34 @@ char *internaltimers(void) {
 	int Count = 0;
 
 	for (int i = 0; i < g_TimerCount; i++) {
+		int rc;
+
 		if (g_Timers[i] == NULL) {
 			continue;
 		}
 
 		Timer[0] = g_Timers[i]->proc;
-		g_asprintf(&Temp1, "%d", g_Timers[i]->timer->GetInterval());
+		rc = asprintf(&Temp1, "%d", g_Timers[i]->timer->GetInterval());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
+
 		Timer[1] = Temp1;
-		g_asprintf(&Temp2, "%d", g_Timers[i]->timer->GetRepeat());
+		rc = asprintf(&Temp2, "%d", g_Timers[i]->timer->GetRepeat());
+
+		if (RcFailed(rc)) {
+			g_Bouncer->Fatal();
+		}
+
 		Timer[2] = Temp2;
+
 		Timer[3] = g_Timers[i]->param ? g_Timers[i]->param : "";
 
 		List[Count++] = Tcl_Merge(4, const_cast<char **>(Timer));
 
-		g_free(Temp1);
-		g_free(Temp2);
+		gfree(Temp1);
+		gfree(Temp2);
 	}
 
 	static char *Out = NULL;
@@ -2014,50 +2104,6 @@ void bncsetgvhost(const char* GVHost) {
 	g_Bouncer->GetConfig()->WriteString("system.ip", GVHost);
 }
 
-const char* getbnchosts(void) {
-	const CVector<char *> *Hosts = g_Bouncer->GetHostAllows();
-
-	int argc = 0;
-	const char** argv = (const char**)malloc(Hosts->GetLength() * sizeof(const char*));
-
-	for (unsigned int i = 0; i < Hosts->GetLength(); i++) {
-		argv[argc++] = Hosts->Get(i);
-	}
-
-	static char* List = NULL;
-
-	if (List)
-		Tcl_Free(List);
-
-	List = Tcl_Merge(argc, const_cast<char **>(argv));
-
-	free(argv);
-
-	return List;
-}
-
-void delbnchost(const char* Host) {
-	RESULT<bool> Result;
-
-	Result = g_Bouncer->RemoveHostAllow(Host, true);
-
-	if (IsError(Result)) {
-		throw GETDESCRIPTION(Result);
-	}
-}
-
-int addbnchost(const char* Host) {
-	RESULT<bool> Result;
-
-	Result = g_Bouncer->AddHostAllow(Host);
-
-	if (IsError(Result)) {
-		throw GETDESCRIPTION(Result);
-	}
-
-	return 1;
-}
-
 bool bncisipblocked(const char* Ip) {
 	CUser* Context = g_Bouncer->GetUser(g_Context);
 
@@ -2072,9 +2118,9 @@ bool bncisipblocked(const char* Ip) {
 
 #ifdef _WIN32
 	Peer.sin_addr.S_un.S_addr = addr;
-#else
+#else /* _WIN32 */
 	Peer.sin_addr.s_addr = addr;
-#endif
+#endif /* _WIN32 */
 
 	return Context->IsIpBlocked((sockaddr *)&Peer);
 }
@@ -2093,17 +2139,13 @@ void bnclogbadlogin(const char* Ip) {
 
 #ifdef _WIN32
 	Peer.sin_addr.S_un.S_addr = addr;
-#else
+#else /* _WIN32 */
 	Peer.sin_addr.s_addr = addr;
-#endif
+#endif /* _WIN32 */
 
 	return Context->LogBadLogin((sockaddr *)&Peer);
 }
 
-bool bnccanhostconnect(const char* Host) {
-	return g_Bouncer->CanHostConnect(Host);
-}
-
 bool bncvalidusername(const char* Name) {
 	return g_Bouncer->IsValidUsername(Name);
 }
@@ -2162,9 +2204,8 @@ void bncaddcommand(const char *Name, const char *Category, const char *Descripti
 		return;
 
 	commandlist_t *List = g_CurrentClient->GetCommandList();
-	const utility_t *Utils = g_Bouncer->GetUtilities();
 
-	Utils->AddCommand(List, Name, Category, Description, HelpText);
+	AddCommand(List, Name, Category, Description, HelpText);
 }
 
 void bncdeletecommand(const char *Name) {
@@ -2180,9 +2221,8 @@ void bncdeletecommand(const char *Name) {
 		return;
 
 	commandlist_t *List = g_CurrentClient->GetCommandList();
-	const utility_t *Utils = g_Bouncer->GetUtilities();
 
-	Utils->DeleteCommand(List, Name);
+	DeleteCommand(List, Name);
 }
 
 void bncsetglobaltag(const char *Tag, const char *Value) {
@@ -2219,58 +2259,6 @@ const char *bncgetglobaltags(void) {
 	return List;
 }
 
-const char *getzoneinfo(const char *Zone) {
-	static char *List = NULL;
-	const CVector<CZoneInformation *> *Zones;
-	char **argv;
-
-	if (List != NULL) {
-		Tcl_Free(List);
-	}
-
-	Zones = g_Bouncer->GetZones();
-
-	if (Zone == NULL) {
-		argv = (char **)malloc(Zones->GetLength() * sizeof(char *));
-
-		for (unsigned int i = 0; i < Zones->GetLength(); i++) {
-			argv[i] = const_cast<char *>(Zones->Get(i)->GetTypeName());
-		}
-
-		List = Tcl_Merge(Zones->GetLength(), const_cast<char **>(argv));
-
-		free(argv);
-	} else {
-		bool Found = false;
-
-		for (unsigned int i = 0; i < Zones->GetLength(); i++) {
-			CZoneInformation *ThisZone = Zones->Get(i);
-
-			if (strcmp(ThisZone->GetTypeName(), Zone) == 0) {
-				argv = (char **)malloc(2 * sizeof(char *));
-				g_asprintf(&(argv[0]), "%d", ThisZone->GetTypeSize());
-				g_asprintf(&(argv[1]), "%d", ThisZone->GetCount());
-
-				List = Tcl_Merge(2, const_cast<char **>(argv));
-
-				g_free(argv[2]);
-				g_free(argv[1]);
-				g_free(argv[0]);
-				free(argv);
-
-				Found = true;
-				break;
-			}
-		}
-
-		if (!Found) {
-			throw "There is no such zone.";
-		}
-	}
-
-	return List;
-}
-
 int hijacksocket(void) {
 	clientdata_t ClientData;
 	CTclClientSocket *TclSocket;
@@ -2349,52 +2337,49 @@ void setchannelsortvalue(int Value) {
 	g_ChannelSortValue = Value;
 }
 
-void TclDnsLookupCallback(void *CookiePtr, hostent *Response) {
-	tcldnsquery_t Cookie = *(tcldnsquery_t *)CookiePtr;
-	const utility_t *Utils;
+void TclDnsLookupCallback(void *CookieRaw, hostent *Response) {
+	tcldnsquery_t *Cookie = (tcldnsquery_t *)CookieRaw;
 	const char *ip, *host, *code;
 	int objc = 5;
 	Tcl_Obj* objv[5];
  
-	Utils = g_Bouncer->GetUtilities();
- 
 	if (Response == NULL) {
 		code = "0";
  
-		if (Cookie.reverse) {
-			ip = Cookie.host;
-			host = Cookie.host;
+		if (Cookie->reverse) {
+			ip = Cookie->host;
+			host = Cookie->host;
 		} else {
-			if (Cookie.ipv6) {
+			if (Cookie->ipv6) {
 				ip = "::0";
 			} else {
 				ip = "0.0.0.0";
 			}
  
-			host = Cookie.host;
+			host = Cookie->host;
 		}
 	} else {
 		code = "1";
  
-		if (Cookie.reverse) {
-			ip = Cookie.host;
+		if (Cookie->reverse) {
+			ip = Cookie->host;
 			host = Response->h_name;
 		} else {
 			const sockaddr *sin;
  
-			sin = Utils->HostEntToSockAddr(Response);
+			sin = HostEntToSockAddr(Response);
  
-			host = Cookie.host;
+			host = Cookie->host;
  
 			if (sin != NULL) {
-				ip = Utils->IpToString(const_cast<sockaddr *>(sin));
+				ip = IpToString(const_cast<sockaddr *>(sin));
 			} else {
 				ip = host;
 			}
 		}
 	}
  
-	objv[0] = Tcl_NewStringObj(Cookie.proc, -1);
+	objv[0] = Tcl_NewStringObj(Cookie->proc, -1);
 	Tcl_IncrRefCount(objv[0]);
  
 	objv[1] = Tcl_NewStringObj(ip, -1);
@@ -2406,8 +2391,8 @@ void TclDnsLookupCallback(void *CookiePtr, hostent *Response) {
 	objv[3] = Tcl_NewStringObj(code, -1);
 	Tcl_IncrRefCount(objv[3]);
  
-	if (Cookie.param) {
-		objv[4] = Tcl_NewStringObj(Cookie.param, -1);
+	if (Cookie->param) {
+		objv[4] = Tcl_NewStringObj(Cookie->param, -1);
 		Tcl_IncrRefCount(objv[4]);
 	} else {
 		objc--;
@@ -2415,7 +2400,7 @@ void TclDnsLookupCallback(void *CookiePtr, hostent *Response) {
  
 	Tcl_EvalObjv(g_Interp, objc, objv, TCL_EVAL_GLOBAL);
  
-	if (Cookie.param) {
+	if (Cookie->param) {
 		Tcl_DecrRefCount(objv[4]);
 	}
  
@@ -2424,21 +2409,20 @@ void TclDnsLookupCallback(void *CookiePtr, hostent *Response) {
 	Tcl_DecrRefCount(objv[1]);
 	Tcl_DecrRefCount(objv[0]);
  
-	free(Cookie.proc);
-	free(Cookie.param);
-	free(Cookie.host);
-	free(CookiePtr);
+	free(Cookie->proc);
+	free(Cookie->param);
+	free(Cookie->host);
+	delete Cookie;
 }
  
 int internaldnslookup(const char *host, const char *tclproc, int reverse, int ipv6, const char *param) {
 	tcldnsquery_t *Cookie;
-	const utility_t *Utils;
  
 	if (tclproc == NULL) {
 		return 1;
 	}
  
-	Cookie = (tcldnsquery_t *)malloc(sizeof(tcldnsquery_t));
+	Cookie = new tcldnsquery_t;
  
 	if (Cookie == NULL) {
 		return 1;
@@ -2463,16 +2447,30 @@ int internaldnslookup(const char *host, const char *tclproc, int reverse, int ip
 	if (!reverse) {
 		Query->GetHostByName(host, af);
 	} else {
-		char sin[MAX_SOCKADDR_LEN];
+		sockaddr_storage sin;
  
-		Utils = g_Bouncer->GetUtilities();
- 
-		if (!Utils->StringToIp(host, af, (sockaddr *)sin, sizeof(sin))) {
+		if (!StringToIp(host, af, (sockaddr *)&sin, sizeof(sin))) {
 			throw "Failed to parse IP address.";
 		}
 
-		Query->GetHostByAddr((sockaddr *)sin);
+		Query->GetHostByAddr((sockaddr *)&sin);
 	}
  
 	return 0;
 }
+
+const char *bncconfigdir(void) {
+	return g_Bouncer->BuildPathConfig("");
+}
+
+const char *bncmoduledir(void) {
+	return g_Bouncer->BuildPathModule("");
+}
+
+const char *bncshareddir(void) {
+    return g_Bouncer->BuildPathShared("");
+}
+
+const char *bncexedir(void) {
+    return g_Bouncer->BuildPathExe("");
+}
