@@ -1,6 +1,6 @@
 /*******************************************************************************
  * shroudBNC - an object-oriented framework for IRC                            *
- * Copyright (C) 2005-2007 Gunnar Beutner                                      *
+ * Copyright (C) 2005-2014 Gunnar Beutner                                      *
  *                                                                             *
  * This program is free software; you can redistribute it and/or               *
  * modify it under the terms of the GNU General Public License                 *
@@ -34,14 +34,14 @@ IMPL_DNSEVENTPROXY(CConnection, AsyncBindIpDnsFinished);
  * @param Role the role of the connection
  */
 CConnection::CConnection(SOCKET Socket, bool SSL, connection_role_e Role) {
-	char Address[MAX_SOCKADDR_LEN];
+	sockaddr_storage Address;
 	socklen_t AddressLength = sizeof(Address);
 
 	SetRole(Role);
 
 	if (Socket != INVALID_SOCKET) {
-		safe_getsockname(Socket, (sockaddr *)Address, &AddressLength);
-		m_Family = ((sockaddr *)Address)->sa_family;
+		getsockname(Socket, (sockaddr *)&Address, &AddressLength);
+		m_Family = Address.ss_family;
 	} else {
 		m_Family = AF_INET;
 	}
@@ -58,10 +58,10 @@ CConnection::CConnection(SOCKET Socket, bool SSL, connection_role_e Role) {
  * @param Port the port
  * @param BindIp the local bind address
  * @param SSL whether to use SSL for the connection
- * @param Family the socket family
+ * @param Family the address family
  */
-CConnection::CConnection(const char *Host, unsigned short Port, const char *BindIp, bool SSL, int Family) {
-	m_Family = Family;
+CConnection::CConnection(const char *Host, unsigned int Port, const char *BindIp, bool SSL, int Family) {
+	m_Family = AF_INET;
 
 	SetRole(Role_Client);
 
@@ -70,20 +70,12 @@ CConnection::CConnection(const char *Host, unsigned short Port, const char *Bind
 	m_Socket = INVALID_SOCKET;
 	m_PortCache = Port;
 	m_BindIpCache = BindIp ? strdup(BindIp) : NULL;
+	m_BindDnsQuery = NULL;
 
 	if (Host != NULL) {
 		m_DnsQuery = new CDnsQuery(this, USE_DNSEVENTPROXY(CConnection, AsyncDnsFinished));
-
 		m_DnsQuery->GetHostByName(Host, Family);
 
-		if (m_BindIpCache != NULL) {
-			m_BindDnsQuery = new CDnsQuery(this, USE_DNSEVENTPROXY(CConnection, AsyncBindIpDnsFinished));
-
-			m_BindDnsQuery->GetHostByName(BindIp, Family);
-		} else {
-			m_BindDnsQuery = NULL;
-		}
-
 		// try to connect.. maybe we already have both addresses
 		AsyncConnect();
 	}
@@ -100,7 +92,6 @@ CConnection::CConnection(const char *Host, unsigned short Port, const char *Bind
 void CConnection::InitConnection(SOCKET Client, bool SSL) {
 	m_Socket = Client;
 
-	m_Locked = false;
 	m_Shutdown = false;
 	m_Timeout = 0;
 
@@ -121,14 +112,14 @@ void CConnection::InitConnection(SOCKET Client, bool SSL) {
 	m_InboundTrafficReset = g_CurrentTime;
 	m_InboundTraffic = 0;
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	m_HasSSL = SSL;
 	m_SSL = NULL;
 
 	if (GetRole() == Role_Server && g_Bouncer->GetSSLContext() == NULL && SSL) {
 		m_HasSSL = false;
 
-		LOGERROR("No SSL server certificate available. Falling back to"
+		g_Bouncer->Log("No SSL server certificate available. Falling back to"
 			" non-SSL mode. This might not work.");
 	}
 #endif
@@ -155,8 +146,8 @@ CConnection::~CConnection(void) {
 	free(m_BindIpCache);
 
 	if (m_Socket != INVALID_SOCKET) {
-		safe_shutdown(m_Socket, SD_BOTH);
-		safe_closesocket(m_Socket);
+		shutdown(m_Socket, SD_BOTH);
+		closesocket(m_Socket);
 	}
 	
 	free(m_HostAddr);
@@ -165,7 +156,7 @@ CConnection::~CConnection(void) {
 	delete m_SendQ;
 	delete m_RecvQ;
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (IsSSL() && m_SSL != NULL) {
 		SSL_free(m_SSL);
 	}
@@ -188,7 +179,7 @@ void CConnection::InitSocket(void) {
 	setsockopt(m_Socket, SOL_SOCKET, SO_LINGER, &optLinger, sizeof(optLinger));
 #endif
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (IsSSL()) {
 		if (m_SSL != NULL) {
 			SSL_free(m_SSL);
@@ -201,7 +192,7 @@ void CConnection::InitSocket(void) {
 		}
 
 		if (m_SSL != NULL) {
-			BIO *Bio = BIO_new_safe_socket(m_Socket, 0);
+			BIO *Bio = BIO_new_socket(m_Socket, 0);
 			SSL_set_bio(m_SSL, Bio, Bio);
 			//SSL_set_fd(m_SSL, m_Socket);
 
@@ -255,8 +246,7 @@ SOCKET CConnection::GetSocket(void) const {
 int CConnection::Read(bool DontProcess) {
 	int ReadResult;
 	static char *Buffer = NULL;
-	static int BufferSize = 0;
-	socklen_t OptLenght = sizeof(BufferSize);
+	const int BufferSize = 8192;
 
 	m_Connected = true;
 
@@ -264,19 +254,15 @@ int CConnection::Read(bool DontProcess) {
 		return 0;
 	}
 
-	if (BufferSize == 0 && safe_getsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char *)&BufferSize, &OptLenght) != 0) {
-		BufferSize = 8192;
-	}
-
 	if (Buffer == NULL) {
 		Buffer = (char *)malloc(BufferSize);
 	}
 
-	CHECK_ALLOC_RESULT(Buffer, malloc) {
+	if (AllocFailed(Buffer)) {
 		return -1;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (IsSSL()) {
 		ReadResult = SSL_read(m_SSL, Buffer, BufferSize);
 
@@ -294,9 +280,12 @@ int CConnection::Read(bool DontProcess) {
 		}
 
 		ERR_print_errors_fp(stdout);
-	} else
+	} else {
+#endif
+		ReadResult = recv(m_Socket, Buffer, BufferSize, 0);
+#ifdef HAVE_LIBSSL
+	}
 #endif
-		ReadResult = safe_recv(m_Socket, Buffer, BufferSize, 0);
 
 	if (ReadResult > 0) {
 		if (g_CurrentTime - m_InboundTrafficReset > 30) {
@@ -318,7 +307,7 @@ int CConnection::Read(bool DontProcess) {
 			return -1;
 		}
 
-		ErrorCode = safe_errno();
+		ErrorCode = errno;
 
 #ifdef _WIN32
 		if (ErrorCode == WSAEWOULDBLOCK) {
@@ -330,7 +319,7 @@ int CConnection::Read(bool DontProcess) {
 		}
 #endif
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 		if (IsSSL()) {
 			SSL_shutdown(m_SSL);
 		}
@@ -353,14 +342,14 @@ int CConnection::Read(bool DontProcess) {
  */
 int CConnection::Write(void) {
 	size_t Size;
-	int ReturnValue;
+	int ReturnValue = 0;
 
 	Size = m_SendQ->GetSize();
 
 	if (Size > 0) {
 		int WriteResult;
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 		if (IsSSL()) {
 			WriteResult = SSL_write(m_SSL, m_SendQ->Peek(), Size);
 
@@ -375,13 +364,13 @@ int CConnection::Write(void) {
 			}
 		} else {
 #endif
-			WriteResult = safe_send(m_Socket, m_SendQ->Peek(), Size, 0);
-#ifdef USESSL
+			WriteResult = send(m_Socket, m_SendQ->Peek(), Size, 0);
+#ifdef HAVE_LIBSSL
 		}
 #endif
 
-		ReturnValue = safe_errno();
-			
+		ReturnValue = errno;
+
 		if (WriteResult > 0) {
 			if (m_Traffic) {
 				m_Traffic->AddOutbound(WriteResult);
@@ -394,15 +383,15 @@ int CConnection::Write(void) {
 	}
 
 	if (m_Shutdown) {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 		if (IsSSL()) {
 			SSL_shutdown(m_SSL);
 		}
 #endif
 
 		if (m_Socket != INVALID_SOCKET) {
-			safe_shutdown(m_Socket, SD_BOTH);
-			safe_closesocket(m_Socket);
+			shutdown(m_Socket, SD_BOTH);
+			closesocket(m_Socket);
 		}
 	}
 
@@ -426,11 +415,9 @@ void CConnection::ProcessBuffer(void) {
 		if (RecvQ[i] == '\n' || (RecvQ[i] == '\r' && Size > i + 1 && RecvQ[i + 1] == '\n')) {
 			char *dupLine = (char *)malloc(&(RecvQ[i]) - Line + 1);
 
-			CHECK_ALLOC_RESULT(dupLine, malloc) {
+			if (AllocFailed(dupLine)) {
 				return;
-			} CHECK_ALLOC_RESULT_END;
-
-			mmark(dupLine);
+			}
 
 			memcpy(dupLine, Line, &(RecvQ[i]) - Line);
 			dupLine[&(RecvQ[i]) - Line] = '\0';
@@ -487,12 +474,13 @@ bool CConnection::ReadLine(char **Out) {
 		char *NewPtr = Pos + 1 + (advance ? 1 : 0);
 
 		Size = NewPtr - old_recvq + 1;
-		*Out = (char *)g_Bouncer->GetUtilities()->Alloc(Size);
-		strmcpy(*Out, m_RecvQ->Read(NewPtr - old_recvq), Size);
+		*Out = (char *)malloc(Size);
 
-		CHECK_ALLOC_RESULT(*Out, strdup) {
+		if (AllocFailed(*Out)) {
 			return false;
-		} CHECK_ALLOC_RESULT_END;
+		}
+
+		strmcpy(*Out, m_RecvQ->Read(NewPtr - old_recvq), Size);
 
 		return true;
 	} else {
@@ -509,9 +497,7 @@ bool CConnection::ReadLine(char **Out) {
  * @param Line the line
  */
 void CConnection::WriteUnformattedLine(const char *Line) {
-	if (!m_Locked && Line != NULL) {
-		m_SendQ->WriteUnformattedLine(Line);
-	}
+	m_SendQ->WriteUnformattedLine(Line);
 }
 
 /**
@@ -531,12 +517,12 @@ void CConnection::WriteLine(const char *Format, ...) {
 	}
 
 	va_start(Marker, Format);
-	vasprintf(&Line, Format, Marker);
+	int rc = vasprintf(&Line, Format, Marker);
 	va_end(Marker);
 
-	CHECK_ALLOC_RESULT(Line, vasprintf) {
+	if (RcFailed(rc)) {
 		return;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	WriteUnformattedLine(Line);
 
@@ -599,7 +585,7 @@ void CConnection::Kill(const char *Error) {
  * written to the socket.
  */
 bool CConnection::HasQueuedData(void) const {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (IsSSL()) {
 		if (SSL_want_write(m_SSL)) {
 			return true;
@@ -651,24 +637,6 @@ void CConnection::Destroy(void) {
 	delete this;
 }
 
-/**
- * Lock
- *
- * Locks the connection object.
- */
-void CConnection::Lock(void) {
-	m_Locked = true;
-}
-
-/**
- * IsLocked
- *
- * Checks whether the connection object is locked.
- */
-bool CConnection::IsLocked(void) const {
-	return m_Locked;
-}
-
 /**
  * Shutdown
  *
@@ -733,7 +701,7 @@ void CConnection::FlushSendQ(void) {
  * Returns whether SSL is enabled for the connection.
  */
 bool CConnection::IsSSL(void) const {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	return m_HasSSL;
 #else
 	return false;
@@ -746,7 +714,7 @@ bool CConnection::IsSSL(void) const {
  * Gets the remote side's certificate.
  */
 const X509 *CConnection::GetPeerCertificate(void) const {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (IsSSL()) {
 		return SSL_get_peer_certificate(m_SSL);
 	}
@@ -771,11 +739,11 @@ int CConnection::SSLVerify(int PreVerifyOk, X509_STORE_CTX *Context) const {
  */
 void CConnection::AsyncConnect(void) {
 	if (m_HostAddr != NULL && (m_BindAddr != NULL || m_BindIpCache == NULL)) {
-		sockaddr *Remote, *Bind = NULL;
+		sockaddr *Remote = NULL, *Bind = NULL;
+		sockaddr_in RemoteV4, BindV4;
+		sockaddr_in6 RemoteV6, BindV6;
 
 		if (m_Family == AF_INET) {
-			sockaddr_in RemoteV4, BindV4;
-
 			memset(&RemoteV4, 0, sizeof(RemoteV4));
 			RemoteV4.sin_family = m_Family;
 			RemoteV4.sin_port = htons(m_PortCache);
@@ -791,10 +759,8 @@ void CConnection::AsyncConnect(void) {
 
 				Bind = (sockaddr *)&BindV4;
 			}
-#ifdef IPV6
-		} else {
-			sockaddr_in6 RemoteV6, BindV6;
-
+#ifdef HAVE_IPV6
+		} else if (m_Family == AF_INET6) {
 			memset(&RemoteV6, 0, sizeof(RemoteV6));
 			RemoteV6.sin6_family = m_Family;
 			RemoteV6.sin6_port = htons(m_PortCache);
@@ -810,19 +776,20 @@ void CConnection::AsyncConnect(void) {
 
 				Bind = (sockaddr *)&BindV6;
 			}
-#endif
+#endif /* HAVE_IPV6 */
+		} else {
+			Error(0);
+
+			return;
 		}
 
-		m_Socket = SocketAndConnectResolved(Remote, Bind);
+		int ErrorCode;
+		m_Socket = SocketAndConnectResolved(Remote, Bind, &ErrorCode);
 
 		free(m_HostAddr);
 		m_HostAddr = NULL;
 
 		if (m_Socket == INVALID_SOCKET) {
-			int ErrorCode;
-
-			ErrorCode = safe_errno();
-
 #ifndef _WIN32
 			if (ErrorCode == 0) {
 				ErrorCode = -1;
@@ -846,32 +813,33 @@ void CConnection::AsyncConnect(void) {
  * @param Response the response
  */
 void CConnection::AsyncDnsFinished(hostent *Response) {
-	if (Response == NULL) {
+	if (Response == NULL || Response->h_addr_list[0] == NULL) {
 		// we cannot destroy the object here as there might still be the other
 		// dns query (bind ip) in the queue which would get destroyed in the
 		// destructor; this causes a crash in the StartMainLoop() function
 		m_LatchedDestruction = true;
-	} else {
-		int Size;
 
-		if (Response->h_addrtype == AF_INET) {
-			Size = sizeof(in_addr);
-		} else {
-			Size = sizeof(in6_addr);
-		}
+		return;
+	 }
 
-		m_HostAddr = (in_addr *)malloc(Size);
+	int Size = INADDR_LEN(Response->h_addrtype);
 
-		CHECK_ALLOC_RESULT(m_HostAddr, malloc) {
-			m_LatchedDestruction = true;
+	m_HostAddr = malloc(Size);
 
-			return;
-		} CHECK_ALLOC_RESULT_END;
-
-		memcpy(m_HostAddr, Response->h_addr_list[0], Size);
-
-		AsyncConnect();
+	if (AllocFailed(m_HostAddr)) {
+		m_LatchedDestruction = true;
+		return;
 	}
+
+	m_Family = Response->h_addrtype;
+	memcpy(m_HostAddr, Response->h_addr_list[0], Size);
+
+	 if (m_BindIpCache != NULL) {
+		m_BindDnsQuery = new CDnsQuery(this, USE_DNSEVENTPROXY(CConnection, AsyncBindIpDnsFinished));
+		m_BindDnsQuery->GetHostByName(m_BindIpCache, Response->h_addrtype);
+	}
+
+	AsyncConnect();
 }
 
 /**
@@ -883,17 +851,14 @@ void CConnection::AsyncDnsFinished(hostent *Response) {
  */
 
 void CConnection::AsyncBindIpDnsFinished(hostent *Response) {
-	if (Response != NULL) {
-		int Size;
+	if (Response != NULL && Response->h_addr_list[0] != NULL) {
+		int Size = INADDR_LEN(Response->h_addrtype);
 
-		if (Response->h_addrtype == AF_INET) {
-			Size = sizeof(in_addr);
-		} else {
-			Size = sizeof(in6_addr);
+		m_BindAddr = malloc(Size);
+
+		if (!AllocFailed(m_BindAddr)) {
+			memcpy(m_BindAddr, Response->h_addr_list[0], Size);
 		}
-
-		m_BindAddr = (in_addr *)malloc(Size);
-		memcpy(m_BindAddr, Response->h_addr_list[0], Size);
 	}
 
 	free(m_BindIpCache);
@@ -921,11 +886,11 @@ bool CConnection::ShouldDestroy(void) const {
  * Returns the remote side's address.
  */
 sockaddr *CConnection::GetRemoteAddress(void) const {
-	static char Result[MAX_SOCKADDR_LEN];
-	socklen_t ResultLength = sizeof(Result);
+	static sockaddr_storage Address;
+	socklen_t AddressLength = sizeof(Address);
 
-	if (m_Socket != INVALID_SOCKET && safe_getpeername(m_Socket, (sockaddr *)Result, &ResultLength) == 0) {
-		return (sockaddr *)Result;
+	if (m_Socket != INVALID_SOCKET && getpeername(m_Socket, (sockaddr *)&Address, &AddressLength) == 0) {
+		return (sockaddr *)&Address;
 	} else {
 		return NULL;
 	}
@@ -937,11 +902,11 @@ sockaddr *CConnection::GetRemoteAddress(void) const {
  * Returns the local side's address.
  */
 sockaddr *CConnection::GetLocalAddress(void) const {
-	static char Result[MAX_SOCKADDR_LEN];
-	socklen_t ResultLength = sizeof(Result);
+	static sockaddr_storage Address;
+	socklen_t AddressLength = sizeof(Address);
 
-	if (m_Socket != INVALID_SOCKET && safe_getsockname(m_Socket, (sockaddr *)Result, &ResultLength) == 0) {
-		return (sockaddr *)Result;
+	if (m_Socket != INVALID_SOCKET && getsockname(m_Socket, (sockaddr *)&Address, &AddressLength) == 0) {
+		return (sockaddr *)&Address;
 	} else {
 		return NULL;
 	}
@@ -963,7 +928,7 @@ bool CConnection::IsConnected(void) {
  */
 size_t CConnection::GetInboundRate(void) {
 	if (g_CurrentTime - m_InboundTrafficReset > 0) {
-		return m_InboundTraffic / (g_CurrentTime - m_InboundTrafficReset);
+		return m_InboundTraffic / (size_t)(g_CurrentTime - m_InboundTrafficReset);
 	} else {
 		return 0;
 	}
@@ -977,9 +942,7 @@ size_t CConnection::GetInboundRate(void) {
  * @param Buffer the new queue
  */
 void CConnection::SetSendQ(CFIFOBuffer *Buffer) {
-	if (m_SendQ != NULL) {
-		delete m_SendQ;
-	}
+	delete m_SendQ;
 
 	m_SendQ = Buffer;
 
@@ -996,9 +959,7 @@ void CConnection::SetSendQ(CFIFOBuffer *Buffer) {
  * @param Buffer the new queue
  */
 void CConnection::SetRecvQ(CFIFOBuffer *Buffer) {
-	if (m_RecvQ != NULL) {
-		delete m_RecvQ;
-	}
+	delete m_RecvQ;
 
 	m_RecvQ = Buffer;
 
@@ -1015,7 +976,7 @@ void CConnection::SetRecvQ(CFIFOBuffer *Buffer) {
  * @param SSLObject ssl object
  */
 void CConnection::SetSSLObject(void *SSLObject) {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (SSLObject == NULL) {
 		m_HasSSL = false;
 	} else {
