@@ -1,6 +1,6 @@
 /*******************************************************************************
  * shroudBNC - an object-oriented framework for IRC                            *
- * Copyright (C) 2005-2007 Gunnar Beutner                                      *
+ * Copyright (C) 2005-2014 Gunnar Beutner                                      *
  *                                                                             *
  * This program is free software; you can redistribute it and/or               *
  * modify it under the terms of the GNU General Public License                 *
@@ -36,12 +36,12 @@ extern time_t g_LastReconnect;
  * @param SSL whether to use SSL
  * @param Family socket family (either AF_INET or AF_INET6)
  */
-CIRCConnection::CIRCConnection(const char *Host, unsigned short Port, CUser *Owner, safe_box_t Box, const char *BindIp, bool SSL, int Family) : CConnection(Host, Port, BindIp, SSL, Family) {
+CIRCConnection::CIRCConnection(const char *Host, unsigned int Port, CUser *Owner,
+		const char *BindIp, bool SSL, int Family) : CConnection(Host, Port, BindIp, SSL, Family) {
 	const char *Ident;
 
 	SetRole(Role_Client);
 	SetOwner(Owner);
-	SetBox(Box);
 
 	g_LastReconnect = g_CurrentTime;
 	m_LastResponse = g_LastReconnect;
@@ -58,33 +58,25 @@ CIRCConnection::CIRCConnection(const char *Host, unsigned short Port, CUser *Own
 
 	m_QueueHigh = new CQueue();
 
-	if (m_QueueHigh == NULL) {
-		LOGERROR("new operator failed. Could not create queue.");
-
+	if (AllocFailed(m_QueueHigh)) {
 		g_Bouncer->Fatal();
 	}
 
 	m_QueueMiddle = new CQueue();
 
-	if (m_QueueMiddle == NULL) {
-		LOGERROR("new operator failed. Could not create queue.");
-
+	if (AllocFailed(m_QueueMiddle)) {
 		g_Bouncer->Fatal();
 	}
 
 	m_QueueLow = new CQueue();
 
-	if (m_QueueLow == NULL) {
-		LOGERROR("new operator failed. Could not create queue.");
-
+	if (AllocFailed(m_QueueLow)) {
 		g_Bouncer->Fatal();
 	}
 
 	m_FloodControl = new CFloodControl();
 
-	if (m_FloodControl == NULL) {
-		LOGERROR("new operator failed. Could not create flood control object.");
-
+	if (AllocFailed(m_FloodControl)) {
 		g_Bouncer->Fatal();
 	}
 
@@ -106,29 +98,26 @@ CIRCConnection::CIRCConnection(const char *Host, unsigned short Port, CUser *Own
 		WriteLine("USER %s \"\" \"fnords\" :%s", Ident, Owner->GetRealname());
 	}
 
-	m_Channels = new CHashtable<CChannel *, false, 16>();
-
-	if (m_Channels == NULL) {
-		LOGERROR("new operator failed. Could not create channel list.");
+	m_Channels = new CHashtable<CChannel *, false>();
 
+	if (AllocFailed(m_Channels)) {
 		g_Bouncer->Fatal();
 	}
 
 	m_Channels->RegisterValueDestructor(DestroyObject<CChannel>);
 
-	m_ISupport = new CHashtable<char *, false, 32>();
-	m_ISupport->RegisterValueDestructor(FreeUString);
-
-	if (m_ISupport == NULL) {
-		LOGERROR("new operator failed. Could not create ISupport object.");
+	m_ISupport = new CHashtable<char *, false>();
 
+	if (AllocFailed(m_ISupport)) {
 		g_Bouncer->Fatal();
 	}
 
-	m_ISupport->Add("CHANMODES", ustrdup("bIe,k,l"));
-	m_ISupport->Add("CHANTYPES", ustrdup("#&+"));
-	m_ISupport->Add("PREFIX", ustrdup("(ov)@+"));
-	m_ISupport->Add("NAMESX", ustrdup(""));
+	m_ISupport->RegisterValueDestructor(FreeString);
+
+	m_ISupport->Add("CHANMODES", strdup("bIe,k,l"));
+	m_ISupport->Add("CHANTYPES", strdup("#&+"));
+	m_ISupport->Add("PREFIX", strdup("(ov)@+"));
+	m_ISupport->Add("NAMESX", strdup(""));
 
 	m_FloodControl->AttachInputQueue(m_QueueHigh, 0);
 	m_FloodControl->AttachInputQueue(m_QueueMiddle, 1);
@@ -137,85 +126,6 @@ CIRCConnection::CIRCConnection(const char *Host, unsigned short Port, CUser *Own
 	m_PingTimer = g_Bouncer->CreateTimer(180, true, IRCPingTimer, this);
 	m_DelayJoinTimer = NULL;
 	m_NickCatchTimer = NULL;
-
-	if (Host == NULL && Box != NULL) {
-		SOCKET Socket;
-		safe_box_t ISupportBox, ChannelsBox;
-		const char *Temp;
-
-		Socket = safe_get_integer(Box, "Socket");
-
-		SetSocket(Socket);
-
-		Temp = safe_get_string(Box, "Nick");
-
-		if (Temp != NULL) {
-			m_CurrentNick = ustrdup(Temp);
-		}
-
-		Temp = safe_get_string(Box, "Server");
-
-		if (Temp != NULL) {
-			m_Server = ustrdup(Temp);
-		}
-
-		Temp = safe_get_string(Box, "ServerVersion");
-
-		if (Temp != NULL) {
-			m_ServerVersion = ustrdup(Temp);
-		}
-
-		Temp = safe_get_string(Box, "ServerFeatures");
-
-		if (Temp != NULL) {
-			m_ServerFeat = ustrdup(Temp);
-		}
-
-		ISupportBox = safe_get_box(Box, "ISupport");
-
-		if (ISupportBox != NULL) {
-			char Name[128];
-			safe_element_t *Previous = NULL;
-
-			while (safe_enumerate(ISupportBox, &Previous, Name, sizeof(Name)) != -1) {
-				const char *Value = safe_get_string(ISupportBox, Name);
-
-				if (Value == NULL) {
-					continue;
-				}
-
-				m_ISupport->Add(Name, ustrdup(Value));
-			}
-		} else {
-			WriteLine("VERSION");
-		}
-
-		ChannelsBox = safe_get_box(Box, "Channels");
-
-		if (ChannelsBox != NULL) {
-			char Name[128];
-			safe_element_t *Previous = NULL;
-
-			Previous = NULL;
-
-			while (safe_enumerate(ChannelsBox, &Previous, Name, sizeof(Name)) != -1) {
-				CChannel* Channel;
-				safe_box_t ChannelBox;
-
-				ChannelBox = safe_get_box(ChannelsBox, Name);
-
-				if (ChannelBox == NULL) {
-					continue;
-				}
-
-				Channel = new CChannel(Name, this, ChannelBox);
-
-				if (Channel != NULL) {
-					m_Channels->Add(Name, Channel);
-				}
-			}
-		}
-	}
 }
 
 /**
@@ -224,15 +134,15 @@ CIRCConnection::CIRCConnection(const char *Host, unsigned short Port, CUser *Own
  * Destructs a connection object.
  */
 CIRCConnection::~CIRCConnection(void) {
-	ufree(m_CurrentNick);
-	ufree(m_Site);
-	ufree(m_Usermodes);
+	free(m_CurrentNick);
+	free(m_Site);
+	free(m_Usermodes);
 
 	delete m_Channels;
 
-	ufree(m_Server);
-	ufree(m_ServerVersion);
-	ufree(m_ServerFeat);
+	free(m_Server);
+	free(m_ServerVersion);
+	free(m_ServerFeat);
 
 	delete m_ISupport;
 
@@ -327,6 +237,8 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			if (User != NULL) {
 				User->SetIdleSince(g_CurrentTime);
 			}
+
+			Channel->AddBacklogLine(argv[0], argv[3]);
 		}
 
 		if (!ModuleEvent(argc, argv)) {
@@ -335,21 +247,21 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 		}
 
 		/* don't log ctcp requests */
-		if (argv[3][0] != '\1' && argv[3][strlen(argv[3]) - 1] != '\1' && Dest != NULL && Nick != NULL && m_CurrentNick != NULL && strcasecmp(Dest, m_CurrentNick) == 0 && strcasecmp(Nick, m_CurrentNick) != 0) {
+		if (argv[3][0] != '\1' && argv[3][strlen(argv[3]) - 1] != '\1' && Dest != NULL &&
+				Nick != NULL && m_CurrentNick != NULL && strcasecmp(Dest, m_CurrentNick) == 0 &&
+				strcasecmp(Nick, m_CurrentNick) != 0) {
 			char *Dup;
 			char *Delim;
 
 			Dup = strdup(Reply);
 
-			if (Dup == NULL) {
-				LOGERROR("strdup() failed.");
-
+			if (AllocFailed(Dup)) {
 				free(Nick);
 
 				return true;
 			}
 
-			Delim = strstr(Dup, "!");
+			Delim = strchr(Dup, '!');
 
 			if (Delim != NULL) {
 				*Delim = '\0';
@@ -367,6 +279,12 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 		UpdateHostHelper(Reply);
 
 		return true;
+	} else if (argc > 3 && hashRaw == hashPrivmsg && Client != NULL) {
+		Channel = GetChannel(argv[2]);
+
+		if (Channel != NULL) {
+			Channel->AddBacklogLine(argv[0], argv[3]);
+		}
 	} else if (argc > 3 && hashRaw == hashNotice && Client == NULL) {
 		const char *Dest = argv[2];
 		char *Nick;
@@ -378,7 +296,9 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 		Nick = ::NickFromHostmask(Reply);
 
 		/* don't log ctcp replies */
-		if (argv[3][0] != '\1' && argv[3][strlen(argv[3]) - 1] != '\1' && Dest != NULL && Nick != NULL && m_CurrentNick != NULL && strcasecmp(Dest, m_CurrentNick) == 0 && strcasecmp(Nick, m_CurrentNick) != 0) {
+		if (argv[3][0] != '\1' && argv[3][strlen(argv[3]) - 1] != '\1' && Dest != NULL &&
+				Nick != NULL && m_CurrentNick != NULL && strcasecmp(Dest, m_CurrentNick) == 0 &&
+				strcasecmp(Nick, m_CurrentNick) != 0) {
 			GetOwner()->Log("%s (notice): %s", Reply, argv[3]);
 		}
 
@@ -400,9 +320,7 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 		if (Channel != NULL) {
 			Nick = NickFromHostmask(Reply);
 
-			if (Nick == NULL) {
-				LOGERROR("NickFromHostmask() failed.");
-
+			if (AllocFailed(Nick)) {
 				return false;
 			}
 
@@ -422,9 +340,7 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			if (Channel != NULL) {
 				Nick = ::NickFromHostmask(Reply);
 
-				if (Nick == NULL) {
-					LOGERROR("NickFromHostmask() failed.");
-
+				if (AllocFailed(Nick)) {
 					return false;
 				}
 
@@ -446,14 +362,12 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			if (Client == NULL) {
 				char *Dup = strdup(Reply);
 
-				if (Dup == NULL) {
-					LOGERROR("strdup() failed. Could not log event.");
-
+				if (AllocFailed(Dup)) {
 					return bRet;
 				}
 
-				char *Delim = strstr(Dup, "!");
-				const char *Host;
+				char *Delim = strchr(Dup, '!');
+				const char *Host = NULL;
 
 				if (Delim) {
 					*Delim = '\0';
@@ -479,97 +393,33 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 	} else if (argc > 2 && iRaw == 1) {
 		if (Client != NULL) {
 			if (strcmp(Client->GetNick(), argv[2]) != 0) {
-				Client->WriteLine(":%s!ident@sbnc NICK :%s", Client->GetNick(), argv[2]);
+				Client->WriteLine(":%s!%s NICK :%s", Client->GetNick(), m_Site ? m_Site : "unknown@unknown.host", argv[2]);
 			}
 		}
 
-		ufree(m_CurrentNick);
-		m_CurrentNick = ustrdup(argv[2]);
+		free(m_CurrentNick);
+		m_CurrentNick = strdup(argv[2]);
 
-		ufree(m_Server);
-		m_Server = ustrdup(Reply);
+		free(m_Server);
+		m_Server = strdup(Reply);
 
-		if (GetBox() != NULL) {
-			safe_put_string(GetBox(), "Nick", argv[2]);
-			safe_put_string(GetBox(), "Server", Reply);
+		const CVector<CModule *> *Modules = g_Bouncer->GetModules();
+
+		for (int i = 0; i < Modules->GetLength(); i++) {
+			(*Modules)[i]->ServerLogon(GetOwner()->GetUsername());
 		}
-	} else if (argc > 2 && hashRaw == hashNick) {
-		if (b_Me) {
-			ufree(m_CurrentNick);
-			m_CurrentNick = ustrdup(argv[2]);
 
-			if (GetBox() != NULL) {
-				safe_put_string(GetBox(), "Nick", argv[2]);
+		if (Client != NULL) {
+			if (strcmp(m_CurrentNick, Client->GetNick()) != 0) {
+				Client->ChangeNick(m_CurrentNick);
 			}
 		}
 
-		Nick = NickFromHostmask(argv[0]);
+		GetOwner()->Log("You were successfully connected to an IRC server.");
+		g_Bouncer->Log("User %s connected to an IRC server.",
+			GetOwner()->GetUsername());
 
-		int i = 0;
-
-		if (!b_Me && GetOwner()->GetClientConnectionMultiplexer() == NULL) {
-			const char *AwayNick = GetOwner()->GetAwayNick();
-
-			if (AwayNick != NULL && strcasecmp(AwayNick, Nick) == 0) {
-				WriteLine("NICK %s", AwayNick);
-			}
-		}
-
-		while (hash_t<CChannel *> *ChannelHash = m_Channels->Iterate(i++)) {
-			ChannelHash->Value->RenameUser(Nick, argv[2]);
-		}
-
-		free(Nick);
-	} else if (argc > 1 && hashRaw == hashQuit) {
-		bool bRet = ModuleEvent(argc, argv);
-
-		Nick = NickFromHostmask(argv[0]);
-
-		unsigned int i = 0;
-
-		while (hash_t<CChannel *> *ChannelHash = m_Channels->Iterate(i++)) {
-			ChannelHash->Value->RemoveUser(Nick);
-		}
-
-		free(Nick);
-
-		return bRet;
-	} else if (argc > 1 && (iRaw == 422 || iRaw == 376)) {
 		int DelayJoin = GetOwner()->GetDelayJoin();
-		if (m_State != State_Connected) {
-			const CVector<CModule *> *Modules = g_Bouncer->GetModules();
-
-			for (unsigned int i = 0; i < Modules->GetLength(); i++) {
-				(*Modules)[i]->ServerLogon(GetOwner()->GetUsername());
-			}
-
-			const char *ClientNick;
-
-			if (Client != NULL) {
-				ClientNick = Client->GetNick();
-
-				if (strcmp(m_CurrentNick, ClientNick) != 0) {
-					Client->ChangeNick(m_CurrentNick);
-				}
-			}
-
-			GetOwner()->Log("You were successfully connected to an IRC server.");
-			g_Bouncer->Log("User %s connected to an IRC server.", GetOwner()->GetUsername());
-
-			if (GetBox() != NULL) {
-				safe_put_integer(GetBox(), "Socket", GetSocket());
-			}
-		}
-
-/*		if (Client != NULL && Client->GetPreviousNick() != NULL && strcmp(Client->GetPreviousNick(), m_CurrentNick) != 0) {
-			const char *Site = GetSite();
-
-			if (Site == NULL) {
-				Site = "unknown@host";
-			}
-
-			Client->WriteLine(":%s!%s NICK %s", Client->GetPreviousNick(), Site, m_CurrentNick);
-		}*/
 
 		if (DelayJoin == 1) {
 			m_DelayJoinTimer = g_Bouncer->CreateTimer(5, false, DelayJoinTimer, this);
@@ -598,6 +448,44 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 		}
 
 		m_State = State_Connected;
+
+	} else if (argc > 2 && hashRaw == hashNick) {
+		if (b_Me) {
+			free(m_CurrentNick);
+			m_CurrentNick = strdup(argv[2]);
+		}
+
+		Nick = NickFromHostmask(argv[0]);
+
+		int i = 0;
+
+		if (!b_Me && GetOwner()->GetClientConnectionMultiplexer() == NULL) {
+			const char *AwayNick = GetOwner()->GetAwayNick();
+
+			if (AwayNick != NULL && strcasecmp(AwayNick, Nick) == 0) {
+				WriteLine("NICK %s", AwayNick);
+			}
+		}
+
+		while (hash_t<CChannel *> *ChannelHash = m_Channels->Iterate(i++)) {
+			ChannelHash->Value->RenameUser(Nick, argv[2]);
+		}
+
+		free(Nick);
+	} else if (argc > 1 && hashRaw == hashQuit) {
+		bool bRet = ModuleEvent(argc, argv);
+
+		Nick = NickFromHostmask(argv[0]);
+
+		int i = 0;
+
+		while (hash_t<CChannel *> *ChannelHash = m_Channels->Iterate(i++)) {
+			ChannelHash->Value->RemoveUser(Nick);
+		}
+
+		free(Nick);
+
+		return bRet;
 	} else if (argc > 1 && strcasecmp(Reply, "ERROR") == 0) {
 		if (strstr(Raw, "throttle") != NULL) {
 			GetOwner()->ScheduleReconnect(120);
@@ -605,29 +493,36 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			GetOwner()->ScheduleReconnect(5);
 		}
 
-		g_Bouncer->LogUser(GetUser(), "Error received for user %s: %s", GetOwner()->GetUsername(), argv[1]);
-	} else if (argc > 3 && iRaw == 465) {
-		g_Bouncer->LogUser(GetUser(), "G/K-line reason for user %s: %s", GetOwner()->GetUsername(), argv[3]);
-	} else if (argc > 5 && iRaw == 351) {
-		ufree(m_ServerVersion);
-		m_ServerVersion = ustrdup(argv[3]);
-
-		ufree(m_ServerFeat);
-		m_ServerFeat = ustrdup(argv[5]);
-
-		if (GetBox() != NULL) {
-			safe_put_string(GetBox(), "ServerVersion", argv[3]);
-			safe_put_string(GetBox(), "ServerFeatures", argv[5]);
+		if (GetCurrentNick() != NULL && GetSite() != NULL) {
+			g_Bouncer->LogUser(GetUser(), "Error received for user %s [%s!%s]: %s",
+				GetOwner()->GetUsername(), GetCurrentNick(), GetSite(), argv[1]);
+		} else {
+			g_Bouncer->LogUser(GetUser(), "Error received for user %s: %s",
+				GetOwner()->GetUsername(), argv[1]);
 		}
-	} else if (argc > 3 && iRaw	== 5) {
+	} else if (argc > 3 && iRaw == 465) {
+		if (GetCurrentNick() != NULL && GetSite() != NULL) {
+			g_Bouncer->LogUser(GetUser(), "G/K-line reason for user %s [%s!%s]: %s",
+				GetOwner()->GetUsername(), GetCurrentNick(), GetSite(), argv[3]);
+		} else {
+			g_Bouncer->LogUser(GetUser(), "G/K-line reason for user %s: %s",
+				GetOwner()->GetUsername(), argv[3]);
+		}
+	} else if (argc > 5 && iRaw == 351) {
+		free(m_ServerVersion);
+		m_ServerVersion = strdup(argv[3]);
+
+		free(m_ServerFeat);
+		m_ServerFeat = strdup(argv[5]);
+	} else if (argc > 3 && iRaw == 5) {
 		for (int i = 3; i < argc - 1; i++) {
 			char *Dup = strdup(argv[i]);
 
-			CHECK_ALLOC_RESULT(Dup, strdup) {
+			if (AllocFailed(Dup)) {
 				return false;
-			} CHECK_ALLOC_RESULT_END;
+			}
 
-			char *Eq = strstr(Dup, "=");
+			char *Eq = strchr(Dup, '=');
 
 			if (strcasecmp(Dup, "NAMESX") == 0) {
 				WriteLine("PROTOCTL NAMESX");
@@ -638,23 +533,13 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			if (Eq) {
 				*Eq = '\0';
 
-				Value = ustrdup(++Eq);
+				Value = strdup(++Eq);
 			} else {
-				Value = ustrdup("");
+				Value = strdup("");
 			}
 
 			m_ISupport->Add(Dup, Value);
 
-			safe_box_t ISupportBox;
-
-			if (GetBox() != NULL) {
-				ISupportBox = safe_put_box(GetBox(), "ISupport");
-
-				if (ISupportBox != NULL) {
-					safe_put_string(ISupportBox, Dup, Value);
-				}
-			}
-
 			free(Dup);
 		}
 	} else if (argc > 4 && iRaw == 324) {
@@ -670,7 +555,7 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 
 		if (Channel != NULL) {
 			Channel->ParseModeChange(argv[0], argv[3], argc - 4, &argv[4]);
-		} else if (strcmp(m_CurrentNick, argv[2]) == 0) {
+		} else if (m_CurrentNick && strcmp(m_CurrentNick, argv[2]) == 0) {
 			bool Flip = true, WasNull;
 			const char *Modes = argv[3];
 			size_t Length = strlen(Modes) + 1;
@@ -680,11 +565,11 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			}
 
 			WasNull = (m_Usermodes != NULL) ? false : true;
-			m_Usermodes = (char *)urealloc(m_Usermodes, Length);
+			m_Usermodes = (char *)realloc(m_Usermodes, Length);
 
-			CHECK_ALLOC_RESULT(m_Usermodes, realloc) {
+			if (AllocFailed(m_Usermodes)) {
 				return false;
-			} CHECK_ALLOC_RESULT_END;
+			}
 
 			if (WasNull) {
 				m_Usermodes[0] = '\0';
@@ -765,17 +650,17 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 
 			nicks = ArgTokenize(argv[5]);
 
-			CHECK_ALLOC_RESULT(nicks, ArgTokenize) {
+			if (AllocFailed(nicks)) {
 				return false;
-			} CHECK_ALLOC_RESULT_END;
+			}
 
 			nickv = ArgToArray(nicks);
 
-			CHECK_ALLOC_RESULT(nickv, ArgToArray) {
+			if (AllocFailed(nickv)) {
 				ArgFree(nicks);
 
 				return false;
-			} CHECK_ALLOC_RESULT_END;
+			}
 
 			int nickc = ArgCount(nicks);
 
@@ -783,13 +668,14 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 				char *Nick = strdup(nickv[i]);
 				char *BaseNick = Nick;
 
-				CHECK_ALLOC_RESULT(Nick, strdup) {
+				if (AllocFailed(Nick)) {
+					ArgFreeArray(nickv);
 					ArgFree(nicks);
 
 					return false;
-				} CHECK_ALLOC_RESULT_END;
+				}
 
-				StrTrim(Nick);
+				StrTrim(Nick, ' ');
 
 				while (IsNickPrefix(*Nick)) {
 					Nick++;
@@ -800,9 +686,9 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 				if (BaseNick != Nick) {
 					Modes = (char *)malloc(Nick - BaseNick + 1);
 
-					CHECK_ALLOC_RESULT(Modes, malloc) {} else {
+					if (!AllocFailed(Modes)) {
 						strmcpy(Modes, BaseNick, Nick - BaseNick + 1);
-					} CHECK_ALLOC_RESULT_END;
+					}
 				}
 
 				Channel->AddUser(Nick, Modes);
@@ -828,12 +714,14 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 		const char *Realname = argv[9];
 		char *Mask;
 
-		asprintf(&Mask, "%s!%s@%s", Nick, Ident, Host);
+		int rc = asprintf(&Mask, "%s!%s@%s", Nick, Ident, Host);
 
-		UpdateHostHelper(Mask);
-		UpdateWhoHelper(Nick, Realname, Server);
+		if (!RcFailed(rc)) {
+			UpdateHostHelper(Mask);
+			UpdateWhoHelper(Nick, Realname, Server);
 
-		free(Mask);
+			free(Mask);
+		}
 	} else if (argc > 6 && iRaw == 367) {
 		Channel = GetChannel(argv[3]);
 
@@ -847,15 +735,17 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 			Channel->SetHasBans();
 		}
 	} else if (argc > 3 && iRaw == 396) {
-		ufree(m_Site);
-		m_Site = ustrdup(argv[3]);
+		free(m_Site);
+		m_Site = strdup(argv[3]);
 
-		if (m_Site == NULL) {
-			LOGERROR("ustrdup() failed.");
-		}
-	} else if (argc > 3 && hashRaw == hashPong && m_Server != NULL && strcasecmp(argv[3], m_Server) == 0 && m_EatPong) {
+		if (AllocFailed(m_Site)) {}
+	} else if (argc > 3 && hashRaw == hashPong && m_Server != NULL && strcasecmp(argv[2], m_Server) == 0 && m_EatPong) {
 		m_EatPong = false;
 
+		return false;
+	} else if (argc > 3 && iRaw == 421) {
+		m_FloodControl->Unplug();
+
 		return false;
 	}
 
@@ -877,7 +767,7 @@ bool CIRCConnection::ParseLineArgV(int argc, const char **argv) {
 bool CIRCConnection::ModuleEvent(int argc, const char **argv) {
 	const CVector<CModule *> *Modules = g_Bouncer->GetModules();
 
-	for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+	for (int i = 0; i < Modules->GetLength(); i++) {
 		if (!(*Modules)[i]->InterceptIRCMessage(this, argc, argv)) {
 			return false;
 		}
@@ -912,25 +802,25 @@ void CIRCConnection::ParseLine(const char *Line) {
 	int argc = ArgCount2(Args);
 
 	if (argv == NULL) {
-		LOGERROR("ArgToArray2 returned NULL. Could not parse line (%s).", Line);
+		g_Bouncer->Log("ArgToArray2 returned NULL. Could not parse line (%s).", Line);
 
 		return;
 	}
 
 	if (ParseLineArgV(argc, argv)) {
 		if (strcasecmp(argv[0], "ping") == 0 && argc > 1) {
-			asprintf(&Out, "PONG :%s", argv[1]);
+			int rc = asprintf(&Out, "PONG :%s", argv[1]);
 
-			CHECK_ALLOC_RESULT(Out, asprintf) {} else {
+			if (!RcFailed(rc)) {
 				m_QueueHigh->QueueItem(Out);
 				free(Out);
-			} CHECK_ALLOC_RESULT_END;
+			}
 
 			if (m_State != State_Connected) {
 				m_State = State_Pong;
 
 				if (GetOwner()->GetClientConnectionMultiplexer() == NULL) {
-					WriteLine("VERSION");
+					WriteUnformattedLine("VERSION");
 				}
 			}
 		} else {
@@ -940,7 +830,11 @@ void CIRCConnection::ParseLine(const char *Line) {
 				CClientConnection *Client = User->GetClientConnectionMultiplexer();
 
 				if (Client != NULL) {
-					Client->WriteLine("%s", Line);
+					if (argc > 2 && strcasecmp(argv[1], "303") == 0) {
+						Client->WriteLine("%s -sBNC", Line);
+					} else {
+						Client->WriteUnformattedLine(Line);
+					}
 				}
 			}
 		}
@@ -974,41 +868,22 @@ const char *CIRCConnection::GetCurrentNick(void) const {
  * @param Channel the channel's name
  */
 CChannel *CIRCConnection::AddChannel(const char *Channel) {
-	CUser *User;
 	CChannel *ChannelObj;
-	safe_box_t ChannelBox = NULL;
 	bool LimitExceeded = false;
 
 	if (g_Bouncer->GetResourceLimit("channels") < m_Channels->GetLength()) {
 		LimitExceeded = true;
 		ChannelObj = NULL;
 	} else {
-		safe_box_t ChannelsBox, ChannelBox = NULL;
-
-		if (GetBox() != NULL) {
-			ChannelsBox = safe_put_box(GetBox(), "Channels");
-
-			if (ChannelsBox != NULL) {
-				ChannelBox = safe_put_box(ChannelsBox, Channel);
-			}
-		}
-
-		ChannelObj = unew CChannel(Channel, this, ChannelBox);
+		ChannelObj = new CChannel(Channel, this);
 	}
 
-	CHECK_ALLOC_RESULT(ChannelObj, unew) {
+	if (AllocFailed(ChannelObj)) {
+		GetUser()->Log("Out of memory. Removing channel (%s).", Channel);
 		WriteLine("PART %s", Channel);
 
-		User = GetUser();
-
-		if (User->MemoryIsLimitExceeded()) {
-			LimitExceeded = true;
-		}
-
-		if (LimitExceeded) {
-			User->Log("Memory/Channel limit exceeded. Removing channel (%s).", Channel);
-		}
-	} CHECK_ALLOC_RESULT_END;
+		return NULL;
+	}
 
 	m_Channels->Add(Channel, ChannelObj);
 
@@ -1056,9 +931,9 @@ void CIRCConnection::UpdateChannelConfig(void) {
 		Size = (Out ? strlen(Out) : 0) + strlen(Chan->Name) + 2;
 		Out = (char *)realloc(Out, Size);
 
-		CHECK_ALLOC_RESULT(Out, realloc) {
+		if (AllocFailed(Out)) {
 			return;
-		} CHECK_ALLOC_RESULT_END;
+		}
 
 		if (!WasNull) {
 			strmcat(Out, ",", Size);
@@ -1129,7 +1004,7 @@ const char *CIRCConnection::GetISupport(const char *Feature) const {
  * @param Value new value for the feature
  */
 void CIRCConnection::SetISupport(const char *Feature, const char *Value) {
-	m_ISupport->Add(Feature, ustrdup(Value));
+	m_ISupport->Add(Feature, strdup(Value));
 }
 
 /**
@@ -1198,7 +1073,7 @@ bool CIRCConnection::IsNickPrefix(char Char) const {
 		return false;
 	}
 
-	for (unsigned int i = 0; i < strlen(Prefixes); i++) {
+	for (size_t i = 0; i < strlen(Prefixes); i++) {
 		if (flip) {
 			if (Prefixes[i] == Char) {
 				return true;
@@ -1242,7 +1117,7 @@ bool CIRCConnection::IsNickMode(char Char) const {
  */
 char CIRCConnection::PrefixForChanMode(char Mode) const {
 	const char *Prefixes = GetISupport("PREFIX");
-	const char *ActualPrefixes = strstr(Prefixes, ")");
+	const char *ActualPrefixes = strchr(Prefixes, ')');
 
 	Prefixes++;
 
@@ -1287,7 +1162,7 @@ const char *CIRCConnection::GetServerFeat(void) const {
  *
  * Returns a hashtable object which contains all 005 replies.
  */
-const CHashtable<char *, false, 32> *CIRCConnection::GetISupportAll(void) const {
+const CHashtable<char *, false> *CIRCConnection::GetISupportAll(void) const {
 	return m_ISupport;
 }
 
@@ -1301,7 +1176,7 @@ const CHashtable<char *, false, 32> *CIRCConnection::GetISupportAll(void) const
  * @param Server the servername for the user
  */
 void CIRCConnection::UpdateWhoHelper(const char *Nick, const char *Realname, const char *Server) {
-	int a = 0, i = 0;
+	int a = 0;
 
 	if (GetOwner()->GetLeanMode() > 0) {
 		return;
@@ -1337,7 +1212,7 @@ void CIRCConnection::UpdateHostHelper(const char *Host) {
 		return;
 	}
 
-	NickEnd = strstr(Host, "!");
+	NickEnd = strchr(Host, '!');
 
 	if (NickEnd == NULL) {
 		return;
@@ -1347,9 +1222,7 @@ void CIRCConnection::UpdateHostHelper(const char *Host) {
 
 	Copy = strdup(Host);
 
-	if (Copy == NULL) {
-		LOGERROR("strdup() failed. Could not update hostmask. (%s)", Host);
-
+	if (AllocFailed(Copy)) {
 		return;
 	}
 
@@ -1360,12 +1233,10 @@ void CIRCConnection::UpdateHostHelper(const char *Host) {
 	Site++;
 
 	if (m_CurrentNick && strcasecmp(Nick, m_CurrentNick) == 0) {
-		ufree(m_Site);
-		m_Site = ustrdup(Site);
+		free(m_Site);
+		m_Site = strdup(Site);
 
-		if (m_Site == NULL) {
-			LOGERROR("ustrdup() failed.");
-		}
+		if (AllocFailed(m_Site)) {}
 	}
 
 	if (GetOwner()->GetLeanMode() > 0) {
@@ -1377,7 +1248,7 @@ void CIRCConnection::UpdateHostHelper(const char *Host) {
 	int i = 0;
 
 	while (hash_t<CChannel *> *Chan = m_Channels->Iterate(i++)) {
-		if (!Chan->Value->HasNames()) {
+		if (!Chan->Value || !Chan->Value->HasNames()) {
 			continue;
 		}
 
@@ -1408,7 +1279,7 @@ CFloodControl *CIRCConnection::GetFloodControl(void) {
  * @param In the line
  */
 void CIRCConnection::WriteUnformattedLine(const char *In) {
-	if (!m_Locked && strlen(In) < 512) {
+	if (strlen(In) < 512) {
 		m_QueueMiddle->QueueItem(In);
 	}
 }
@@ -1463,9 +1334,9 @@ void CIRCConnection::JoinChannels(void) {
 
 		DupChannels = strdup(Channels);
 
-		CHECK_ALLOC_RESULT(DupChannels, strdup) {
+		if (AllocFailed(DupChannels)) {
 			return;
-		} CHECK_ALLOC_RESULT_END;
+		}
 
 		Channel = strtok(DupChannels, ",");
 
@@ -1486,20 +1357,18 @@ void CIRCConnection::JoinChannels(void) {
 					Size = strlen(Channel) + 1;
 					ChanList = (char *)malloc(Size);
 
-					CHECK_ALLOC_RESULT(ChanList, malloc) {
+					if (AllocFailed(ChanList)) {
 						free(DupChannels);
 
 						return;
-					} CHECK_ALLOC_RESULT_END;
+					}
 
 					strmcpy(ChanList, Channel, Size);
 				} else {
 					Size = strlen(ChanList) + 1 + strlen(Channel) + 2;
 					newChanList = (char *)realloc(ChanList, Size);
 
-					if (newChanList == NULL) {
-						LOGERROR("realloc() failed. Could not join channel.");
-
+					if (AllocFailed(newChanList)) {
 						continue;
 					}
 
@@ -1599,7 +1468,7 @@ bool IRCPingTimer(time_t Now, void *IRCConnection) {
  *
  * Returns a hashtable containing all channels which the user is currently in.
  */
-CHashtable<CChannel *, false, 16> *CIRCConnection::GetChannels(void) {
+CHashtable<CChannel *, false> *CIRCConnection::GetChannels(void) {
 	return m_Channels;
 }
 
@@ -1609,28 +1478,6 @@ CHashtable<CChannel *, false, 16> *CIRCConnection::GetChannels(void) {
  * Returns the site (ident\@host) for the IRC connection.
  */
 const char *CIRCConnection::GetSite(void) {
-	char *Out;
-
-	if (m_Site == NULL) {
-		asprintf(&Out, "%s@unknown.host", GetOwner()->GetUsername());
-
-		CHECK_ALLOC_RESULT(Out, asprintf) {
-			return NULL;
-		} CHECK_ALLOC_RESULT_END;
-
-		char *Foo = mstrdup("Foo", GETUSER());
-
-		m_Site = ustrdup(Out);
-
-		free(Out);
-
-		CHECK_ALLOC_RESULT(m_Site, ustrdup) {
-			return NULL;
-		} CHECK_ALLOC_RESULT_END;
-
-		mmark(m_Site);
-	}
-
 	return m_Site;
 }
 
@@ -1643,7 +1490,7 @@ const char *CIRCConnection::GetSite(void) {
  * @param Context the X509 context
  */
 int CIRCConnection::SSLVerify(int PreVerifyOk, X509_STORE_CTX *Context) const {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (GetOwner()->GetClientConnectionMultiplexer() != NULL) {
 		GetOwner()->GetClientConnectionMultiplexer()->Privmsg(Context->cert->name);
 	}
@@ -1675,8 +1522,12 @@ void CIRCConnection::AsyncDnsFinished(hostent *Response) {
  * @param Response the reponse from the DNS server
  */
 void CIRCConnection::AsyncBindIpDnsFinished(hostent *Response) {
-	if (Response == NULL && GetOwner() != NULL) {
-		g_Bouncer->LogUser(GetOwner(), "DNS request (vhost) for user %s failed.", GetOwner()->GetUsername());
+	if ((Response == NULL || Response->h_addr_list[0] == NULL) && GetOwner() != NULL) {
+		g_Bouncer->LogUser(GetOwner(), "DNS request (vhost) for user %s failed. Cancelling connection attempt.", GetOwner()->GetUsername());
+
+		m_LatchedDestruction = true;
+
+		return;
 	}
 
 	CConnection::AsyncBindIpDnsFinished(Response);
@@ -1695,134 +1546,6 @@ void CIRCConnection::Destroy(void) {
 	delete this;
 }
 
-/**
- * Thaw
- *
- * De-persists an IRC connection object.
- *
- * @param Box the box
- * @param Owner the owner of the IRC connection
- */
-/*RESULT<CIRCConnection *> CIRCConnection::Thaw(safe_box_t Box, CUser *Owner) {
-	SOCKET Socket;
-	CIRCConnection *Connection;
-	const char *Temp;
-
-	if (Box == NULL) {
-		THROW(CIRCConnection *, Generic_InvalidArgument, "Box cannot be NULL.");
-	}
-
-	Socket = safe_get_integer(Box, "Socket");
-
-	Connection = new CIRCConnection(Socket, Owner, Box, false);
-
-	Temp = safe_get_string(Box, "Nick");
-
-	if (Temp != NULL) {
-		Connection->m_CurrentNick = nstrdup(Temp);
-	}
-
-	Temp = safe_get_string(Box, "Server");
-
-	if (Temp != NULL) {
-		Connection->m_Server = nstrdup(Temp);
-	}
-
-	Temp = safe_get_string(Box, "ServerVersion");
-
-	if (Temp != NULL){
-		Connection->m_ServerVersion = nstrdup(Temp);
-	}
-
-	Temp = safe_get_string(Box, "ServerFeatures");
-
-	if (Temp != NULL) {
-		Connection->m_ServerFeat = nstrdup(Temp);
-	}
-
-	safe_box_t ISupportBox = safe_get_box(Box, "ISupportBox");
-	safe_element_t *Previous = NULL;
-
-	if (ISupportBox != NULL) {
-		char Name[128];
-
-		Connection->m_ISupport = new CHashtable<char *, false, 32>();
-
-		while (safe_enumerate(ISupportBox, &Previous, Name, sizeof(Name)) != -1) {
-			const char *Value = safe_get_string(ISupportBox, Name);
-
-			if (Value == NULL) {
-				continue;
-			}
-
-			Connection->m_ISupport->Add(Name, mstrdup(Value, Owner));
-		}
-	}
-
-	Connection->WriteLine("VERSION");
-
-	safe_box_t ChannelsBox = safe_get_box(Box, "Channels");
-
-	if (ChannelsBox != NULL) {
-		char Name[128];
-
-		Previous = NULL;
-
-		while (safe_enumerate(ChannelsBox, &Previous, Name, sizeof(Name)) != -1) {
-			CChannel* Channel;
-			safe_box_t ChannelBox;
-
-			ChannelBox = safe_get_box(ChannelsBox, Name);
-
-			if (ChannelBox == NULL) {
-				continue;
-			}
-
-			Channel = new CChannel(Name, Connection, ChannelBox);
-
-			if (Channel != NULL) {
-				Connection->m_Channels->Add(Channel->GetName(), Channel);
-			}
-		}
-	}
-
-*//*	delete Connection->m_FloodControl;
-
-	Connection->m_FloodControl = new CFloodControl();
-
-	TempBox = Box->ReadBox("~irc.queuehigh");
-
-	if (TempBox != NULL) {
-		delete Connection->m_QueueHigh;
-
-		Connection->m_QueueHigh = CQueue::Thaw(TempBox);
-	}
-
-	Connection->m_FloodControl->AttachInputQueue(Connection->m_QueueHigh, 0);
-
-	TempBox = Box->ReadBox("~irc.queuemiddle");
-
-	if (TempBox != NULL) {
-		delete Connection->m_QueueMiddle;
-
-		Connection->m_QueueMiddle = CQueue::Thaw(TempBox);
-	}
-
-	Connection->m_FloodControl->AttachInputQueue(Connection->m_QueueMiddle, 0);
-
-	TempBox = Box->ReadBox("~irc.queuelow");
-
-	if (TempBox != NULL) {
-		delete Connection->m_QueueLow;
-
-		Connection->m_QueueLow = CQueue::Thaw(TempBox);
-	}
-
-	Connection->m_FloodControl->AttachInputQueue(Connection->m_QueueLow, 0);*//*
-
-	RETURN(CIRCConnection *, Connection);
-}*/
-
 /**
  * Kill
  *
@@ -1859,7 +1582,7 @@ char CIRCConnection::GetHighestUserFlag(const char *Modes) const {
 		return '\0';
 	}
 
-	for (unsigned int i = 0; i < strlen(Prefixes); i++) {
+	for (size_t i = 0; i < strlen(Prefixes); i++) {
 		if (Flip == false) {
 			if (Prefixes[i] == ')') {
 				Flip = true;
@@ -1896,18 +1619,25 @@ void CIRCConnection::Error(int ErrorValue) {
 #endif
 	}
 
-	if (m_State == State_Connecting && GetOwner() != NULL) {
-		if (!IsConnected()) {
-			if (ErrorMsg == NULL || ErrorMsg[0] == '\0') {
-				g_Bouncer->LogUser(GetOwner(), "An error occurred while connecting for user %s.", GetOwner()->GetUsername());
+	if (GetOwner() != NULL) {
+		if (ErrorMsg == NULL || ErrorMsg[0] == '\0') {
+			if (GetCurrentNick() != NULL && GetSite() != NULL) {
+				g_Bouncer->LogUser(GetOwner(), "User '%s' [%s!%s] was disconnected "
+					"from the IRC server.", GetOwner()->GetUsername(),
+					GetCurrentNick(), GetSite());
 			} else {
-				g_Bouncer->LogUser(GetOwner(), "An error occurred while connecting for user %s: %s", GetOwner()->GetUsername(), ErrorMsg);
+				g_Bouncer->LogUser(GetOwner(), "User '%s' was disconnected "
+					"from the IRC server.", GetOwner()->GetUsername());
 			}
 		} else {
-			if (ErrorMsg == NULL || ErrorMsg[0] == '\0') {
-				g_Bouncer->LogUser(GetOwner(), "An error occurred while processing a connection for user %s.", GetOwner()->GetUsername());
+			if (GetCurrentNick() != NULL && GetSite() != NULL) {
+				g_Bouncer->LogUser(GetOwner(), "User '%s' [%s!%s] was disconnected "
+					"from the IRC server: %s", GetOwner()->GetUsername(),
+					GetCurrentNick(), GetSite(), ErrorMsg);
 			} else {
-				g_Bouncer->LogUser(GetOwner(), "An error occurred while processing a connection for user %s: %s", GetOwner()->GetUsername(), ErrorMsg);
+				g_Bouncer->LogUser(GetOwner(), "User '%s' was disconnected "
+					"from the IRC server: %s", GetOwner()->GetUsername(),
+					ErrorMsg);
 			}
 		}
 	}
@@ -1965,3 +1695,12 @@ bool NickCatchTimer(time_t Now, void *IRCConnection) {
 
 	return false;
 }
+
+/**
+ * GetState
+ *
+ * Returns the state of the IRC connection.
+ */
+int CIRCConnection::GetState(void) {
+	return m_State;
+}
