@@ -1,6 +1,6 @@
 /*******************************************************************************
  * shroudBNC - an object-oriented framework for IRC                            *
- * Copyright (C) 2005-2007 Gunnar Beutner                                      *
+ * Copyright (C) 2005-2014 Gunnar Beutner                                      *
  *                                                                             *
  * This program is free software; you can redistribute it and/or               *
  * modify it under the terms of the GNU General Public License                 *
@@ -30,39 +30,36 @@ CTimer *g_ReconnectTimer = NULL;
  *
  * @param Name the name of the user
  */
-CUser::CUser(const char *Name, safe_box_t Box) {
+CUser::CUser(const char *Name) {
 	char *Out;
+#ifdef HAVE_LIBSSL
 	X509 *Cert;
 	FILE *ClientCert;
-	safe_box_t TrafficClientStatsBox = NULL, TrafficIRCStatsBox = NULL;
-
-	SetBox(Box);
-
-	m_ManagedMemory = 0;
-	m_MemoryManager = NULL;
+#endif
+	int rc;
 
 	m_PrimaryClient = NULL;
 	m_ClientMultiplexer = new CClientConnectionMultiplexer(this);
 	m_IRC = NULL;
-	m_Name = ustrdup(Name);
+	m_Name = strdup(Name);
 
-	CHECK_ALLOC_RESULT(m_Name, strdup) {
+	if (AllocFailed(m_Name)) {
 		g_Bouncer->Fatal();
-	} CHECK_ALLOC_RESULT_END;
+	}
 
-	asprintf(&Out, "users/%s.conf", Name);
+	rc = asprintf(&Out, "users/%s.conf", Name);
 
-	CHECK_ALLOC_RESULT(Out, asprintf) {
+	if (RcFailed(rc)) {
 		g_Bouncer->Fatal();
-	} CHECK_ALLOC_RESULT_END;
+	}
 
-	m_Config = g_Bouncer->CreateConfigObject(Out, this);
+	m_Config = new CConfig(Out, this);
 
 	free(Out);
 
-	CHECK_ALLOC_RESULT(m_Config, new) {
+	if (AllocFailed(m_Config)) {
 		g_Bouncer->Fatal();
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	CacheInitialize(m_ConfigCache, m_Config, "user.");
 
@@ -70,41 +67,37 @@ CUser::CUser(const char *Name, safe_box_t Box) {
 
 	m_ReconnectTime = 0;
 	m_LastReconnect = 0;
+	m_NextProtocolFamily = AF_UNSPEC;
 
-	asprintf(&Out, "users/%s.log", Name);
+	rc = asprintf(&Out, "users/%s.log", Name);
 
-	CHECK_ALLOC_RESULT(Out, asprintf) {
+	if (RcFailed(rc)) {
 		g_Bouncer->Fatal();
-	} CHECK_ALLOC_RESULT_END;
+	}
 
-	m_Log = new CLog(g_Bouncer->BuildPath(Out));
+	m_Log = new CLog(g_Bouncer->BuildPathConfig(Out));
 
 	free(Out);
 
-	CHECK_ALLOC_RESULT(m_Log, new) {
+	if (AllocFailed(m_Log)) {
 		g_Bouncer->Fatal();
-	} CHECK_ALLOC_RESULT_END;
-
-	if (GetBox() != NULL) {
-		TrafficClientStatsBox = safe_put_box(GetBox(), "TrafficClientStats");
-		TrafficIRCStatsBox = safe_put_box(GetBox(), "TrafficIRCStats");
 	}
 
-	m_ClientStats = new CTrafficStats(TrafficClientStatsBox);
-	m_IRCStats = new CTrafficStats(TrafficIRCStatsBox);
+	m_ClientStats = new CTrafficStats();
+	m_IRCStats = new CTrafficStats();
 
 	m_Keys = new CKeyring(m_Config, this);
 
 	m_BadLoginPulse = new CTimer(200, true, BadLoginTimer, this);
 
-#ifdef USESSL
-	asprintf(&Out, "users/%s.pem", Name);
+#ifdef HAVE_LIBSSL
+	rc = asprintf(&Out, "users/%s.pem", Name);
 
-	CHECK_ALLOC_RESULT(Out, asprintf) {
+	if (RcFailed(rc)) {
 		g_Bouncer->Fatal();
-	} CHECK_ALLOC_RESULT_END;
+	}
 
-	ClientCert = fopen(g_Bouncer->BuildPath(Out), "r");
+	ClientCert = fopen(g_Bouncer->BuildPathConfig(Out), "r");
 
 	free(Out);
 
@@ -125,47 +118,6 @@ CUser::CUser(const char *Name, safe_box_t Box) {
 	if (IsAdmin()) {
 		g_Bouncer->GetAdminUsers()->Insert(this);
 	}
-
-	if (Box != NULL) {
-		safe_box_t IRCBox = safe_get_box(Box, "IRC");
-
-		if (IRCBox != NULL) {
-			CIRCConnection *IRC;
-
-			IRC = new CIRCConnection(NULL, 0, this, IRCBox, NULL);
-
-			if (IRC != NULL) {
-				SetIRCConnection(IRC);
-			}
-		}
-
-		safe_box_t ClientsBox = safe_get_box(Box, "Clients");
-
-		if (ClientsBox != NULL) {
-			safe_element_t *PreviousClient = NULL;
-			char ClientName[128];
-
-			safe_set_ro(ClientsBox, 1);
-
-			while (safe_enumerate(ClientsBox, &PreviousClient, ClientName, sizeof(ClientName)) != -1) {
-				CClientConnection *Client;
-
-				safe_box_t ClientBox = safe_get_box(ClientsBox, ClientName);
-
-				Client = new CClientConnection(INVALID_SOCKET, ClientBox);
-
-				if (Client != NULL) {
-					AddClientConnection(Client);
-
-					if (IsAdmin()) {
-						Client->Privmsg("shroudBNC seems to have died unexpectedly. However the parent process was able to successfully resurrect it.");
-					}
-				}
-			}
-
-			safe_set_ro(ClientsBox, 0);
-		}	
-	}
 }
 
 /**
@@ -174,10 +126,6 @@ CUser::CUser(const char *Name, safe_box_t Box) {
  * Destructs a user object.
  */
 CUser::~CUser(void) {
-	if (m_MemoryManager != NULL) {
-		m_MemoryManager->RealManager = NULL;
-	}
-
 	m_ClientMultiplexer->Kill("Removing user.");
 
 	if (m_IRC != NULL) {
@@ -192,14 +140,14 @@ CUser::~CUser(void) {
 
 	delete m_Keys;
 
-	ufree(m_Name);
+	free(m_Name);
 
 	if (m_BadLoginPulse != NULL) {
 		m_BadLoginPulse->Destroy();
 	}
 
-#ifdef USESSL
-	for (unsigned int i = 0; i < m_ClientCertificates.GetLength(); i++) {
+#ifdef HAVE_LIBSSL
+	for (int i = 0; i < m_ClientCertificates.GetLength(); i++) {
 		X509_free(m_ClientCertificates[i]);
 	}
 #endif
@@ -217,7 +165,7 @@ void CUser::LoadEvent(void) {
 
 	Modules = g_Bouncer->GetModules();
 
-	for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+	for (int i = 0; i < Modules->GetLength(); i++) {
 		(*Modules)[i]->UserLoad(m_Name);
 	}
 }
@@ -266,9 +214,10 @@ void CUser::Attach(CClientConnection *Client) {
 	char *Out;
 	const char *Reason, *AutoModes, *IrcNick, *MotdText;
 	CLog *Motd;
-	unsigned int i;
+	int i;
 	bool Added = false;
 	bool FirstClient;
+	int rc;
 
 	if (IsLocked()) {
 		Reason = GetSuspendReason();
@@ -276,11 +225,9 @@ void CUser::Attach(CClientConnection *Client) {
 		if (Reason == NULL) {
 			Client->Kill("*** You cannot attach to this user.");
 		} else {
-			asprintf(&Out, "*** Your account is suspended. Reason: %s", Reason);
-
-			if (Out == NULL) {
-				LOGERROR("asprintf() failed.");
+			rc = asprintf(&Out, "*** Your account is suspended. Reason: %s", Reason);
 
+			if (RcFailed(rc)) {
 				Client->Kill("*** Your account is suspended.");
 			} else {
 				Client->Kill(Out);
@@ -331,7 +278,8 @@ void CUser::Attach(CClientConnection *Client) {
 			Client->ParseLine("SYNTH VERSION-FORCEREPLY");
 
 			if (m_IRC->GetUsermodes() != NULL) {
-				Client->WriteLine(":%s!%s@%s MODE %s +%s", IrcNick, GetUsername(), Client->GetPeerName(), IrcNick, m_IRC->GetUsermodes());
+				const char *Site = m_IRC->GetSite();
+				Client->WriteLine(":%s!%s MODE %s +%s", IrcNick, Site ? Site : "unknown@unknown.host", IrcNick, m_IRC->GetUsermodes());
 			}
 
 			AddClientConnection(Client);
@@ -341,11 +289,11 @@ void CUser::Attach(CClientConnection *Client) {
 
 			Channels = (CChannel **)malloc(sizeof(CChannel *) * m_IRC->GetChannels()->GetLength());
 
-			CHECK_ALLOC_RESULT(Channels, malloc) {
-				return;
-			} CHECK_ALLOC_RESULT_END;
+			if (AllocFailed(Channels)) {
+				delete Motd;
 
-			mmark(Channels);
+				return;
+			}
 
 			i = 0;
 			while (hash_t<CChannel *> *ChannelHash = m_IRC->GetChannels()->Iterate(i++)) {
@@ -376,40 +324,46 @@ void CUser::Attach(CClientConnection *Client) {
 				SortFunction = ChannelTSCompare;
 			}
 
-			qsort(Channels, m_IRC->GetChannels()->GetLength(), sizeof(CChannel *), SortFunction);
+			qsort(Channels, m_IRC->GetChannels()->GetLength(), sizeof(Channels[0]), SortFunction);
+
+			const char *Site = m_IRC->GetSite();
 
 			i = 0;
 			for (i = 0; i < m_IRC->GetChannels()->GetLength(); i++) {
-				Client->WriteLine(":%s!%s@%s JOIN %s", m_IRC->GetCurrentNick(), GetUsername(), Client->GetPeerName(), Channels[i]->GetName());
+				Client->WriteLine(":%s!%s JOIN %s", m_IRC->GetCurrentNick(), Site ? Site : "unknown@unknown.host", Channels[i]->GetName());
 
-				asprintf(&Out, "TOPIC %s", Channels[i]->GetName());
-
-				if (Out == NULL) {
-					LOGERROR("asprintf() failed.");
+				rc = asprintf(&Out, "TOPIC %s", Channels[i]->GetName());
 
+				if (RcFailed(rc)) {
 					Client->Kill("Internal error.");
 				} else {
 					Client->ParseLine(Out);
 					free(Out);
 				}
 
-				asprintf(&Out, "NAMES %s", Channels[i]->GetName());
-
-				if (Out == NULL) {
-					LOGERROR("asprintf() failed.");
+				rc = asprintf(&Out, "NAMES %s", Channels[i]->GetName());
 
+				if (RcFailed(rc)) {
 					Client->Kill("Internal error.");
 				} else {
 					Client->ParseLine(Out);
 					free(Out);
 				}
+
+				if (GetAutoBacklog() != NULL && strcasecmp(GetAutoBacklog(), "off") != 0) {
+					Channels[i]->PlayBacklog(Client);
+				}
 			}
 
 			free(Channels);
 		}
 	} else {
+		Client->WriteLine(":shroudbnc.info 001 %s :Welcome to the Internet Relay Network %s", Client->GetNick(), Client->GetNick());
+
 		if (!Motd->IsEmpty()) {
 			Motd->PlayToUser(Client, Log_Motd);
+		} else {
+			Client->WriteLine(":shroudbnc.info 422 %s :MOTD File is missing", Client->GetNick());
 		}
 
 		if (IsQuitted() != 2) {
@@ -417,6 +371,8 @@ void CUser::Attach(CClientConnection *Client) {
 		}
 	}
 
+	delete Motd;
+
 	if (!Added) {
 		AddClientConnection(Client);
 	}
@@ -424,11 +380,9 @@ void CUser::Attach(CClientConnection *Client) {
 	MotdText = g_Bouncer->GetMotd();
 
 	if (MotdText != NULL) {
-		asprintf(&Out, "Message of the day: %s", MotdText);
-		
-		if (Out == NULL) {
-			LOGERROR("asprintf() failed.");
-		} else {
+		rc = asprintf(&Out, "Message of the day: %s", MotdText);
+
+		if (!RcFailed(rc)) {
 			Client->Privmsg(Out);
 
 			free(Out);
@@ -437,10 +391,15 @@ void CUser::Attach(CClientConnection *Client) {
 
 	if (m_IRC == NULL) {
 		if (GetServer() == NULL) {
-			Client->Privmsg("You haven't set an IRC server yet. Use /sbnc set server <Hostname> <Port> to do that now.");
-			Client->Privmsg("Use /sbnc help to see a list of available commands.");
+			Client->Privmsg("You haven't set an IRC server yet. Use "
+				"/msg -sBNC set server <Hostname> <Port> to do that now.");
+			Client->Privmsg("Use /msg -sBNC help to see a list of available commands.");
 		} else if (IsQuitted() == 2) {
-			Client->Privmsg("You are not connected to an irc server. Use /sbnc jump to reconnect now.");
+			Client->Privmsg("You are not connected to an IRC server. Use "
+				"/msg -sBNC jump to reconnect now.");
+		} else {
+			Client->Privmsg("The connection to the IRC server was lost. "
+				"You will be reconnected shortly.");
 		}
 	}
 
@@ -456,7 +415,8 @@ void CUser::Attach(CClientConnection *Client) {
  *
  * @param Password a password
  */
-bool CUser::CheckPassword(const char *Password) const {
+bool CUser::CheckPassword(const char *Password) {
+	const char *OriginalPassword = Password;
 	const char *RealPass = CacheGetString(m_ConfigCache, password);
 
 	if (RealPass == NULL || Password == NULL || strlen(Password) == 0) {
@@ -470,6 +430,18 @@ bool CUser::CheckPassword(const char *Password) const {
 	if (strcmp(RealPass, Password) == 0) {
 		return true;
 	} else {
+		if (g_Bouncer->GetMD5()) {
+			Password = UtilMd5(OriginalPassword, SaltFromHash(RealPass), true);
+
+			if (strcmp(RealPass, Password) == 0) {
+				SetPassword(OriginalPassword);
+
+				g_Bouncer->Log("Updated broken password for user '%s'.", m_Name);
+
+				return true;
+			}
+		}
+
 		return false;
 	}
 }
@@ -519,13 +491,7 @@ const char *CUser::GetRealname(void) const {
 	const char *Realname = CacheGetString(m_ConfigCache, realname);
 
 	if (Realname == NULL) {
-		Realname = g_Bouncer->GetDefaultRealName();
-
-		if (Realname != NULL) {
-			return Realname;
-		} else {
-			return "shroudBNC User";
-		}
+		return "shroudBNC User";
 	} else {
 		return Realname;
 	}
@@ -566,18 +532,18 @@ void CUser::Simulate(const char *Command, CClientConnection *FakeClient) {
 
 	CommandDup = strdup(Command);
 
-	CHECK_ALLOC_RESULT(CommandDup, strdup) {
+	if (AllocFailed(CommandDup)) {
 		return;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	if (FakeClient == NULL) {
-		FakeClient = new CClientConnection(INVALID_SOCKET, NULL);
+		FakeClient = new CClientConnection(INVALID_SOCKET);
 
-		CHECK_ALLOC_RESULT(FakeClient, new) {
+		if (AllocFailed(FakeClient)) {
 			free(CommandDup);
 
 			return;
-		} CHECK_ALLOC_RESULT_END;
+		}
 	}
 
 	OldOwner = FakeClient->GetOwner();
@@ -613,7 +579,7 @@ void CUser::Simulate(const char *Command, CClientConnection *FakeClient) {
  * @param Client the client connection
  */
 bool CUser::IsRegisteredClientConnection(CClientConnection *Client) {
-	for (unsigned int i = 0; i < m_Clients.GetLength(); i++) {
+	for (int i = 0; i < m_Clients.GetLength(); i++) {
 		if (m_Clients[i].Client == Client) {
 			return true;
 		}
@@ -629,8 +595,7 @@ bool CUser::IsRegisteredClientConnection(CClientConnection *Client) {
  */
 void CUser::Reconnect(void) {
 	const char *Server;
-	int Port;
-	safe_box_t IRCBox = NULL;
+	int Port, i;
 
 	if (m_IRC != NULL) {
 		m_IRC->Kill("Reconnecting.");
@@ -647,10 +612,17 @@ void CUser::Reconnect(void) {
 		return;
 	}
 
-	if (GetIPv6()) {
-		g_Bouncer->LogUser(this, "Trying to reconnect to [%s]:%d for user %s", Server, Port, m_Name);
-	} else {
-		g_Bouncer->LogUser(this, "Trying to reconnect to %s:%d for user %s", Server, Port, m_Name);
+	g_Bouncer->LogUser(this, "Trying to reconnect to [%s]:%d for user %s", Server, Port, m_Name);
+
+	i = 0;
+	while (hash_t<CUser *> *UserHash = g_Bouncer->GetUsers()->Iterate(i++)) {
+		CIRCConnection *IRC;
+
+		IRC = UserHash->Value->GetIRCConnection();
+
+		if (IRC != NULL && IRC->GetState() != State_Connected) {
+			IRC->Kill("Timed out.");
+		}
 	}
 
 	m_LastReconnect = g_CurrentTime;
@@ -671,20 +643,14 @@ void CUser::Reconnect(void) {
 		g_Bouncer->SetIdent(m_Name);
 	}
 
-	if (GetBox() != NULL) {
-		IRCBox = safe_put_box(GetBox(), "IRC");
+	CIRCConnection *Connection = new CIRCConnection(Server, Port, this, BindIp, GetSSL(), m_NextProtocolFamily);
+
+	if (AllocFailed(Connection)) {
+		return;
 	}
 
-	CIRCConnection *Connection = new CIRCConnection(Server, Port, this, IRCBox, BindIp, GetSSL(), GetIPv6() ? AF_INET6 : AF_INET);
-
-	CHECK_ALLOC_RESULT(Connection, new) {
-		return;
-	} CHECK_ALLOC_RESULT_END;
-
 	SetIRCConnection(Connection);
 
-	g_Bouncer->Log("Connection initialized for user %s. Waiting for response...", GetUsername());
-
 	RescheduleReconnectTimer();
 }
 
@@ -704,7 +670,9 @@ bool CUser::ShouldReconnect(void) const {
 		Interval = 25;
 	}
 
-	if (m_IRC == NULL && m_ReconnectTime <= g_CurrentTime && (IsAdmin() || g_CurrentTime - m_LastReconnect > 120) && g_CurrentTime - g_LastReconnect > Interval && IsQuitted() == 0) {
+	if (m_IRC == NULL && m_ReconnectTime <= g_CurrentTime &&
+			(IsAdmin() || g_CurrentTime - m_LastReconnect > 120) &&
+			g_CurrentTime - g_LastReconnect > Interval && IsQuitted() == 0) {
 		return true;
 	} else {
 		return false;
@@ -725,7 +693,7 @@ void CUser::ScheduleReconnect(int Delay) {
 		return;
 	}
 
-	CacheSetInteger(m_ConfigCache, quitted, 0);
+	UnmarkQuitted();
 
 	MaxDelay = Delay;
 	Interval = g_Bouncer->GetInterval();
@@ -750,13 +718,13 @@ void CUser::ScheduleReconnect(int Delay) {
 
 	if (GetServer() != NULL && GetClientConnectionMultiplexer() != NULL) {
 		char *Out;
-		asprintf(&Out, "Scheduled reconnect in %d seconds.", m_ReconnectTime - g_CurrentTime);
+		int rc = asprintf(&Out, "Scheduled reconnect in %d seconds.", (int)(m_ReconnectTime - g_CurrentTime));
 
-		CHECK_ALLOC_RESULT(Out, asprintf) {} else {
+		if (!RcFailed(rc)) {
 			GetClientConnectionMultiplexer()->Privmsg(Out);
 
 			free(Out);
-		} CHECK_ALLOC_RESULT_END;
+		}
 	}
 }
 
@@ -767,7 +735,7 @@ void CUser::ScheduleReconnect(int Delay) {
  */
 unsigned int CUser::GetIRCUptime(void) const {
 	if (m_IRC != NULL) {
-		return g_CurrentTime - m_LastReconnect;
+		return (unsigned int)(g_CurrentTime - m_LastReconnect);
 	} else {
 		return 0;
 	}
@@ -795,18 +763,18 @@ void CUser::Log(const char *Format, ...) {
 	va_list marker;
 
 	va_start(marker, Format);
-	vasprintf(&Out, Format, marker);
+	int rc = vasprintf(&Out, Format, marker);
 	va_end(marker);
 
-	CHECK_ALLOC_RESULT(Out, vasprintf) {} else {
+	if (!RcFailed(rc)) {
 		if (GetClientConnectionMultiplexer() == NULL) {
-			m_Log->WriteLine(FormatTime(g_CurrentTime), "%s", Out);
+			m_Log->WriteLine("%s", Out);
 		} else {
 			GetClientConnectionMultiplexer()->Privmsg(Out);
 		}
 
 		free(Out);
-	} CHECK_ALLOC_RESULT_END;
+	}
 }
 
 /**
@@ -840,7 +808,6 @@ void CUser::SetIRCConnection(CIRCConnection *IRC) {
 	bool WasNull;
 
 	if (GetClientConnectionMultiplexer() != NULL && m_IRC != NULL) {
-//		GetClientConnection()->SetPreviousNick(m_IRC->GetCurrentNick());
 		GetClientConnectionMultiplexer()->SetNick(m_IRC->GetCurrentNick());
 	}
 
@@ -849,6 +816,14 @@ void CUser::SetIRCConnection(CIRCConnection *IRC) {
 	} else {
 		WasNull = false;
 
+		if (m_IRC->GetState() == State_Connecting) {
+			if (m_NextProtocolFamily == AF_INET) {
+				m_NextProtocolFamily = AF_UNSPEC;
+			} else {
+				m_NextProtocolFamily = AF_INET;
+			}
+		}
+
 		m_IRC->SetOwner(NULL);
 	}
 
@@ -858,15 +833,15 @@ void CUser::SetIRCConnection(CIRCConnection *IRC) {
 	Modules = g_Bouncer->GetModules();
 
 	if (IRC == NULL && !WasNull) {
-		for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+		for (int i = 0; i < Modules->GetLength(); i++) {
 			(*Modules)[i]->ServerDisconnect(GetUsername());
 		}
 
 		CClientConnection *Client = GetClientConnectionMultiplexer();;
 
 		if (Client != NULL) {
-			CHashtable<CChannel *, false, 16> *Channels;
-			unsigned int i;
+			CHashtable<CChannel *, false> *Channels;
+			int i;
 			hash_t<CChannel *> *ChannelHash;
 
 			Channels = OldIRC->GetChannels();
@@ -877,9 +852,15 @@ void CUser::SetIRCConnection(CIRCConnection *IRC) {
 			}
 		}
 
-		g_Bouncer->LogUser(this, "User %s disconnected from the server.", GetUsername());
+		if (OldIRC->GetSite() != NULL) {
+			g_Bouncer->LogUser(this, "User %s [%s!%s] disconnected from the server.",
+				GetUsername(), OldIRC->GetCurrentNick(), OldIRC->GetSite());
+		} else {
+			g_Bouncer->LogUser(this, "User %s disconnected from the server.",
+				GetUsername());
+		}
 	} else if (IRC) {
-		for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+		for (int i = 0; i < Modules->GetLength(); i++) {
 			(*Modules)[i]->ServerConnect(GetUsername());
 		}
 
@@ -900,10 +881,10 @@ void CUser::SetIRCConnection(CIRCConnection *IRC) {
 void CUser::AddClientConnection(CClientConnection *Client, bool Silent) {
 	sockaddr *Remote;
 	client_t ClientT;
-	unsigned int i;
+	int i;
 	const CVector<CModule *> *Modules;
 	char *Info;
-	client_t OldestClient;
+	client_t OldestClient = {};
 	time_t ThisTimestamp;
 
 	ThisTimestamp = g_CurrentTime;
@@ -923,7 +904,10 @@ void CUser::AddClientConnection(CClientConnection *Client, bool Silent) {
 			}
 		}
 
-		OldestClient.Client->Kill("Another client logged in. Your client has been disconnected because it was the oldest existing client connection.");
+		assert(OldestClient.Client != NULL);
+
+		OldestClient.Client->Kill("Another client logged in. Your client has been disconnected "
+			"because it was the oldest existing client connection.");
 	}
 
 	Client->SetOwner(this);
@@ -931,9 +915,10 @@ void CUser::AddClientConnection(CClientConnection *Client, bool Silent) {
 	Remote = Client->GetRemoteAddress();
 
 	if (!Silent) {
-		g_Bouncer->Log("User %s logged on (from %s[%s]).", GetUsername(), Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
+		g_Bouncer->Log("User %s logged on (from %s[%s]).", GetUsername(),
+			Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
 
-		CacheSetInteger(m_ConfigCache, seen, g_CurrentTime);
+		CacheSetInteger(m_ConfigCache, seen, (int)g_CurrentTime);
 	}
 
 	ClientT.Creation = g_CurrentTime;
@@ -947,14 +932,6 @@ void CUser::AddClientConnection(CClientConnection *Client, bool Silent) {
 
 	m_PrimaryClient = Client;
 
-	if (GetBox() != NULL) {
-		safe_box_t ClientsBox = safe_put_box(GetBox(), "Clients");
-
-		if (ClientsBox != NULL) {
-			safe_move(ClientsBox, Client->GetBox(), NULL);
-		}
-	}
-
 	Client->SetTrafficStats(m_ClientStats);
 
 	if (!Silent) {
@@ -964,13 +941,15 @@ void CUser::AddClientConnection(CClientConnection *Client, bool Silent) {
 			(*Modules)[i]->AttachClient(Client);
 		}
 
-		asprintf(&Info, "Another client logged in from %s[%s]. The new client has been set as the primary client for this account.", Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
+		int rc = asprintf(&Info, "Another client logged in from %s[%s]. The new client "
+			"has been set as the primary client for this account.",
+			Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
 
-		CHECK_ALLOC_RESULT(Info, asprintf) {
+		if (RcFailed(rc)) {
 			return;
-		} CHECK_ALLOC_RESULT_END;
+		}
 
-		for (unsigned int i = 0; i < m_Clients.GetLength(); i++) {
+		for (int i = 0; i < m_Clients.GetLength(); i++) {
 			if (m_Clients[i].Client != Client) {
 				m_Clients[i].Client->Privmsg(Info);
 			}
@@ -989,11 +968,11 @@ void CUser::AddClientConnection(CClientConnection *Client, bool Silent) {
  * @param Silent whether to silently remove the client
  */
 void CUser::RemoveClientConnection(CClientConnection *Client, bool Silent) {
-	const char *AwayMessage, *DropModes, *AwayNick, *AwayText, *Timestamp;
+	const char *AwayMessage, *DropModes, *AwayNick, *AwayText;
 	hash_t<CChannel *> *Channel;
 	const CVector<CModule *> *Modules;
-	unsigned int i;
-	int a;
+	int i;
+	int a, rc;
 	client_t *BestClient;
 	sockaddr *Remote;
 	char *InfoPrimary, *Info;
@@ -1002,19 +981,21 @@ void CUser::RemoveClientConnection(CClientConnection *Client, bool Silent) {
 	LastClient = (m_Clients.GetLength() == 1);
 
 	if (!Silent) {
-		char *Plural = "s";
+		const char *Plural = "s";
 
 		if (m_Clients.GetLength() - 1 == 1) {
 			Plural = "";
 		}
 
 		if (Client->GetQuitReason() != NULL) {
-			g_Bouncer->Log("User %s logged off. %d remaining client%s for this user. (%s)", GetUsername(), m_Clients.GetLength() - 1, Plural, Client->GetQuitReason());
+			g_Bouncer->Log("User %s logged off. %d remaining client%s for this user. (%s)",
+				 GetUsername(), m_Clients.GetLength() - 1, Plural, Client->GetQuitReason());
 		} else {
-			g_Bouncer->Log("User %s logged off. %d remaining client%s for this user.", GetUsername(), m_Clients.GetLength() - 1, Plural);
+			g_Bouncer->Log("User %s logged off. %d remaining client%s for this user.",
+				GetUsername(), m_Clients.GetLength() - 1, Plural);
 		}
 
-		CacheSetInteger(m_ConfigCache, seen, g_CurrentTime);
+		CacheSetInteger(m_ConfigCache, seen, (int)g_CurrentTime);
 	}
 
 	if (!Silent && m_IRC != NULL && m_Clients.GetLength() == 1) {
@@ -1061,13 +1042,7 @@ void CUser::RemoveClientConnection(CClientConnection *Client, bool Silent) {
 		AwayText = CacheGetString(m_ConfigCache, away);
 
 		if (AwayText != NULL && LastClient) {
-			if (!GetAppendTimestamp()) {
-				m_IRC->WriteLine("AWAY :%s", AwayText);
-			} else {
-				Timestamp = FormatTime(g_CurrentTime);
-
-				m_IRC->WriteLine("AWAY :%s (Away since %s)", AwayText, Timestamp);
-			}
+			m_IRC->WriteLine("AWAY :%s", AwayText);
 		}
 	}
 
@@ -1088,21 +1063,24 @@ void CUser::RemoveClientConnection(CClientConnection *Client, bool Silent) {
 	Remote = Client->GetRemoteAddress();
 
 	if (!Silent) {
-		asprintf(&InfoPrimary, "Another client logged off from %s[%s]. Your client has been set as the primary client for this account.", Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
+		rc = asprintf(&InfoPrimary, "Another client logged off from %s[%s]. Your client "
+			"has been set as the primary client for this account.",
+			Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
 
-		CHECK_ALLOC_RESULT(InfoPrimary, asprintf) {
+		if (RcFailed(rc)) {
 			return;
-		} CHECK_ALLOC_RESULT_END;
+		}
 
-		asprintf(&Info, "Another client logged off from %s[%s].", Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
+		rc = asprintf(&Info, "Another client logged off from %s[%s].",
+			Client->GetPeerName(), (Remote != NULL) ? IpToString(Remote) : "unknown");
 
-		CHECK_ALLOC_RESULT(Info, asprintf) {
+		if (RcFailed(rc)) {
 			free(Info);
 
 			return;
-		} CHECK_ALLOC_RESULT_END;
+		}
 
-		for (unsigned int i = 0; i < m_Clients.GetLength(); i++) {
+		for (int i = 0; i < m_Clients.GetLength(); i++) {
 			if (m_Clients[i].Client == m_PrimaryClient) {
 				m_Clients[i].Client->Privmsg(InfoPrimary);
 			} else {
@@ -1268,6 +1246,16 @@ void CUser::MarkQuitted(bool RequireManualJump) {
 	CacheSetInteger(m_ConfigCache, quitted, RequireManualJump ? 2 : 1);
 }
 
+/**
+ * UnmarkQuitted
+ *
+ * Tells the bouncer to automatically try to connect to a server
+ * when the user is not already connected to one.
+ */
+void CUser::UnmarkQuitted(void) {
+	CacheSetInteger(m_ConfigCache, quitted, 0);
+}
+
 /**
  * IsQuitted
  *
@@ -1287,7 +1275,7 @@ int CUser::IsQuitted(void) const {
 void CUser::LogBadLogin(sockaddr *Peer) {
 	badlogin_t BadLogin;
 
-	for (unsigned int i = 0; i < m_BadLogins.GetLength(); i++) {
+	for (int i = 0; i < m_BadLogins.GetLength(); i++) {
 		if (CompareAddress(m_BadLogins[i].Address, Peer) == 0 && m_BadLogins[i].Count < 3) {
 			m_BadLogins[i].Count++;
 
@@ -1296,13 +1284,12 @@ void CUser::LogBadLogin(sockaddr *Peer) {
 	}
 
 	BadLogin.Count = 1;
-	BadLogin.Address = (sockaddr *)umalloc(SOCKADDR_LEN(Peer->sa_family));
+	BadLogin.Address = (sockaddr *)malloc(SOCKADDR_LEN(Peer->sa_family));
 
-	CHECK_ALLOC_RESULT(BadLogin.Address, umalloc) {
+	if (AllocFailed(BadLogin.Address)) {
 		return;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
-	mmark(BadLogin.Address);
 	memcpy(BadLogin.Address, Peer, SOCKADDR_LEN(Peer->sa_family));
 
 	m_BadLogins.Insert(BadLogin);
@@ -1316,7 +1303,7 @@ void CUser::LogBadLogin(sockaddr *Peer) {
  * @param Peer the IP address
  */
 bool CUser::IsIpBlocked(sockaddr *Peer) const {
-	for (unsigned int i = 0; i < m_BadLogins.GetLength(); i++) {
+	for (int i = 0; i < m_BadLogins.GetLength(); i++) {
 		if (CompareAddress(m_BadLogins[i].Address, Peer) == 0) {
 			if (m_BadLogins[i].Count > 2) {
 				return true;
@@ -1340,7 +1327,7 @@ void CUser::BadLoginPulse(void) {
 			m_BadLogins[i].Count--;
 
 			if (m_BadLogins[i].Count <= 0) {
-				ufree(m_BadLogins[i].Address);
+				free(m_BadLogins[i].Address);
 				m_BadLogins.Remove(i);
 			}
 		}
@@ -1617,7 +1604,7 @@ void CUser::SetDropModes(const char *DropModes) {
  * Returns the user's list of client certificates.
  */
 const CVector<X509 *> *CUser::GetClientCertificates(void) const {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	return &m_ClientCertificates;
 #else
 	return NULL;
@@ -1632,10 +1619,10 @@ const CVector<X509 *> *CUser::GetClientCertificates(void) const {
  * @param Certificate the certificate
  */
 bool CUser::AddClientCertificate(const X509 *Certificate) {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	X509 *DuplicateCertificate;
 
-	for (unsigned int i = 0; i < m_ClientCertificates.GetLength(); i++) {
+	for (int i = 0; i < m_ClientCertificates.GetLength(); i++) {
 		if (X509_cmp(m_ClientCertificates[i], Certificate) == 0) {
 			return true;
 		}
@@ -1659,8 +1646,8 @@ bool CUser::AddClientCertificate(const X509 *Certificate) {
  * @param Certificate the certificate
  */
 bool CUser::RemoveClientCertificate(const X509 *Certificate) {
-#ifdef USESSL
-	for (unsigned int i = 0; i < m_ClientCertificates.GetLength(); i++) {
+#ifdef HAVE_LIBSSL
+	for (int i = 0; i < m_ClientCertificates.GetLength(); i++) {
 		if (X509_cmp(m_ClientCertificates[i], Certificate) == 0) {
 			X509_free(m_ClientCertificates[i]);
 
@@ -1674,7 +1661,7 @@ bool CUser::RemoveClientCertificate(const X509 *Certificate) {
 	return false;
 }
 
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 /**
  * PersistCertificates
  *
@@ -1685,13 +1672,18 @@ bool CUser::PersistCertificates(void) {
 	const char *Filename;
 	FILE *CertFile;
 
-	asprintf(&TempFilename, "users/%s.pem", m_Name);
-	Filename = g_Bouncer->BuildPath(TempFilename);
+	int rc = asprintf(&TempFilename, "users/%s.pem", m_Name);
+
+	if (RcFailed(rc)) {
+		return false;
+	}
+
+	Filename = g_Bouncer->BuildPathConfig(TempFilename);
 	free(TempFilename);
 
-	CHECK_ALLOC_RESULT(Filename, asprintf) {
+	if (AllocFailed(Filename)) {
 		return false;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	if (m_ClientCertificates.GetLength() == 0) {
 		unlink(Filename);
@@ -1700,11 +1692,11 @@ bool CUser::PersistCertificates(void) {
 
 		SetPermissions(Filename, S_IRUSR | S_IWUSR);
 
-		CHECK_ALLOC_RESULT(CertFile, fopen) {
+		if (AllocFailed(CertFile)) {
 			return false;
-		} CHECK_ALLOC_RESULT_END;
+		}
 
-		for (unsigned int i = 0; i < m_ClientCertificates.GetLength(); i++) {
+		for (int i = 0; i < m_ClientCertificates.GetLength(); i++) {
 			PEM_write_X509(CertFile, m_ClientCertificates[i]);
 			fprintf(CertFile, "\n");
 		}
@@ -1725,8 +1717,8 @@ bool CUser::PersistCertificates(void) {
  * @param Certificate a certificate
  */
 bool CUser::FindClientCertificate(const X509 *Certificate) const {
-#ifdef USESSL
-	for (unsigned int i = 0; i < m_ClientCertificates.GetLength(); i++) {
+#ifdef HAVE_LIBSSL
+	for (int i = 0; i < m_ClientCertificates.GetLength(); i++) {
 		if (X509_cmp(m_ClientCertificates[i], Certificate) == 0) {
 			return true;
 		}
@@ -1742,7 +1734,7 @@ bool CUser::FindClientCertificate(const X509 *Certificate) const {
  * Returns whether the user is using SSL for IRC connections.
  */
 bool CUser::GetSSL(void) const {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (CacheGetInteger(m_ConfigCache, ssl) != 0) {
 		return true;
 	} else {
@@ -1761,7 +1753,7 @@ bool CUser::GetSSL(void) const {
  * @param SSL a boolean flag
  */
 void CUser::SetSSL(bool SSL) {
-#ifdef USESSL
+#ifdef HAVE_LIBSSL
 	if (SSL) {
 		CacheSetInteger(m_ConfigCache, ssl, 1);
 	} else {
@@ -1805,11 +1797,11 @@ const char *CUser::GetTagString(const char *Tag) const {
 		return NULL;
 	}
 
-	asprintf(&Setting, "tag.%s", Tag);
+	int rc = asprintf(&Setting, "tag.%s", Tag);
 
-	CHECK_ALLOC_RESULT(Setting, asprintf) {
+	if (RcFailed(rc)) {
 		return NULL;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	Value = m_Config->ReadString(Setting);
 
@@ -1852,21 +1844,21 @@ bool CUser::SetTagString(const char *Tag, const char *Value) {
 		return false;
 	}
 
-	asprintf(&Setting, "tag.%s", Tag);
+	int rc = asprintf(&Setting, "tag.%s", Tag);
 
-	CHECK_ALLOC_RESULT(Setting, asprintf) {
+	if (RcFailed(rc)) {
 		return false;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	Modules = g_Bouncer->GetModules();
 
-	for (unsigned int i = 0; i < Modules->GetLength(); i++) {
+	for (int i = 0; i < Modules->GetLength(); i++) {
 		(*Modules)[i]->UserTagModified(Tag, Value);
 	}
 
 	ReturnValue = m_Config->WriteString(Setting, Value);
 
-	Modules = g_Bouncer->GetModules();
+	free(Setting);
 
 	return ReturnValue;
 }
@@ -1883,11 +1875,11 @@ bool CUser::SetTagInteger(const char *Tag, int Value) {
 	bool ReturnValue;
 	char *StringValue;
 
-	asprintf(&StringValue, "%d", Value);
+	int rc = asprintf(&StringValue, "%d", Value);
 
-	CHECK_ALLOC_RESULT(StringValue, asprintf) {
+	if (RcFailed(rc)) {
 		return false;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	ReturnValue = SetTagString(Tag, StringValue);
 
@@ -1922,102 +1914,6 @@ const char *CUser::GetTagName(int Index) const {
 	return NULL;
 }
 
-/**
- * GetIPv6
- *
- * Returns whether the user is using IPv6 for IRC connections.
- */
-bool CUser::GetIPv6(void) const {
-	if (CacheGetInteger(m_ConfigCache, ipv6) != 0) {
-		return true;
-	} else {
-		return false;
-	}
-}
-
-/**
- * SetIPv6
- *
- * Sets whether the user should use IPv6 for connections to IRC servers.
- *
- * @param IPv6 a boolean flag
- */
-void CUser::SetIPv6(bool IPv6) {
-	CacheSetInteger(m_ConfigCache, ipv6, IPv6 ? 1 : 0);
-}
-
-/**
- * FormatTime
- *
- * Formats a timestamp while considering the user's GMT offset.
- *
- * @param Timestamp the timestamp
- */
-const char *CUser::FormatTime(time_t Timestamp, const char *Format) const {
-	tm *Time;
-	static char Buffer[128];
-
-	Timestamp -= GetGmtOffset() * 60;
-	Time = gmtime(&Timestamp);
-
-	if (Time->tm_isdst <= 0) {
-		Timestamp += 3600;
-		Time = gmtime(&Timestamp);
-	}
-
-	if (Format == NULL) {
-		Format = "%c";
-	}
-
-	strftime(Buffer, sizeof(Buffer), Format, Time);
-
-	return Buffer;
-}
-
-/**
- * SetGmtOffset
- *
- * Sets the user's GMT offset (in minutes).
- *
- * @param Offset the new offset
- */
-void CUser::SetGmtOffset(int Offset) {
-	char *Value;
-
-	asprintf(&Value, "%d", Offset % (60 * 24));
-
-	CHECK_ALLOC_RESULT(Value, asprintf) {
-		return;
-	} CHECK_ALLOC_RESULT_END;
-
-	CacheSetString(m_ConfigCache, tz, Value);
-
-	free(Value);
-}
-
-/**
- * GetGmtOffset
- *
- * Returns the user's GMT offset (in minutes).
- */
-int CUser::GetGmtOffset(void) const {
-	const char *Offset;
-
-	Offset = CacheGetString(m_ConfigCache, tz);
-
-	if (Offset == NULL) {
-		struct tm tm;
-		time_t gmt;
-
-		tm = *gmtime(&g_CurrentTime);
-		gmt = mktime(&tm);
-
-		return -(g_CurrentTime - gmt) / 60;
-	} else {
-		return atoi(Offset);
-	}
-}
-
 /**
  * SetSystemNotices
  *
@@ -2083,43 +1979,6 @@ unsigned int CUser::GetLeanMode(void) {
 	return CacheGetInteger(m_ConfigCache, lean);
 }
 
-bool CUser::MemoryAddBytes(size_t Bytes) {
-/*	if (rand() < RAND_MAX / 6) {
-		return false;
-	}*/
-
-	if (m_ManagedMemory + Bytes > g_Bouncer->GetResourceLimit("memory")) {
-		return false;
-	}
-
-	m_ManagedMemory += Bytes;
-
-	return true;
-}
-
-void CUser::MemoryRemoveBytes(size_t Bytes) {
-	m_ManagedMemory -= Bytes;
-}
-
-size_t CUser::MemoryGetSize(void) {
-	return m_ManagedMemory;
-}
-
-size_t CUser::MemoryGetLimit(void) {
-	return g_Bouncer->GetResourceLimit("memory");
-}
-
-mmanager_t *CUser::MemoryGetManager(void) {
-	if (m_MemoryManager == NULL) {
-		m_MemoryManager = (mmanager_t *)malloc(sizeof(mmanager_t));
-
-		m_MemoryManager->RealManager = this;
-		m_MemoryManager->ReferenceCount = 0;
-	}
-
-	return m_MemoryManager;
-}
-
 const char *CUser::SimulateWithResult(const char *Command) {
 	static CFakeClient *FakeClient = NULL;
 
@@ -2133,10 +1992,10 @@ const char *CUser::SimulateWithResult(const char *Command) {
 }
 
 bool GlobalUserReconnectTimer(time_t Now, void *Null) {
-	int i = 0;
+	int i = rand() % g_Bouncer->GetUsers()->GetLength();
 
 	while (hash_t<CUser *> *UserHash = g_Bouncer->GetUsers()->Iterate(i++)) {
-		if (UserHash->Value->ShouldReconnect() && g_Bouncer->GetStatus() == STATUS_RUN) {
+		if (UserHash->Value->ShouldReconnect() && g_Bouncer->GetStatus() == Status_Running) {
 			UserHash->Value->Reconnect();
 
 			break;
@@ -2158,9 +2017,11 @@ void CUser::RescheduleReconnectTimer(void) {
 
 	ReconnectTime = g_ReconnectTimer->GetNextCall();
 
-	if (g_Bouncer->GetStatus() == STATUS_RUN) {
+	if (g_Bouncer->GetStatus() == Status_Running) {
 		while (hash_t<CUser *> *UserHash = g_Bouncer->GetUsers()->Iterate(i++)) {
-			if (UserHash->Value->m_ReconnectTime >= g_CurrentTime && UserHash->Value->m_ReconnectTime < ReconnectTime && UserHash->Value->GetIRCConnection() == NULL) {
+			if (UserHash->Value->m_ReconnectTime >= g_CurrentTime &&
+					UserHash->Value->m_ReconnectTime < ReconnectTime &&
+					UserHash->Value->GetIRCConnection() == NULL) {
 				ReconnectTime = UserHash->Value->m_ReconnectTime;
 			} else if (UserHash->Value->ShouldReconnect()) {
 				UserHash->Value->Reconnect();
@@ -2171,18 +2032,6 @@ void CUser::RescheduleReconnectTimer(void) {
 	g_ReconnectTimer->Reschedule(ReconnectTime);
 }
 
-void CUser::SetAppendTimestamp(bool Value) {
-	CacheSetInteger(m_ConfigCache, ts, Value ? 1 : 0);
-}
-
-bool CUser::GetAppendTimestamp(void) {
-	if (CacheGetInteger(m_ConfigCache, ts)) {
-		return true;
-	} else {
-		return false;
-	}
-}
-
 void CUser::SetUseQuitReason(bool Value) {
 	CacheSetInteger(m_ConfigCache, quitaway, Value ? 1 : 0);
 }
@@ -2202,3 +2051,11 @@ void CUser::SetChannelSortMode(const char *Mode) {
 const char *CUser::GetChannelSortMode(void) const {
 	return CacheGetString(m_ConfigCache, channelsort);
 }
+
+void CUser::SetAutoBacklog(const char *Value) {
+	CacheSetString(m_ConfigCache, autobacklog, Value);
+}
+
+const char *CUser::GetAutoBacklog(void) {
+	return CacheGetString(m_ConfigCache, autobacklog);
+}
