@@ -1,6 +1,6 @@
 /*******************************************************************************
  * shroudBNC - an object-oriented framework for IRC                            *
- * Copyright (C) 2005-2007 Gunnar Beutner                                      *
+ * Copyright (C) 2005-2014 Gunnar Beutner                                      *
  *                                                                             *
  * This program is free software; you can redistribute it and/or               *
  * modify it under the terms of the GNU General Public License                 *
@@ -20,8 +20,7 @@
 #include "StdAfx.h"
 
 extern "C" {
-	#include "md5/global.h"
-	#include "md5/md5.h"
+	#include "../third-party/md5/md5.h"
 }
 
 /**
@@ -45,9 +44,9 @@ const char *ArgTokenize(const char *Data) {
 	Size = LengthData + 2;
 	Copy = (char *)malloc(Size);
 
-	CHECK_ALLOC_RESULT(Copy, malloc) {
+	if (AllocFailed(Copy)) {
 		return NULL;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	strmcpy(Copy, Data, Size);
 	Copy[LengthData + 1] = '\0';
@@ -141,9 +140,9 @@ const char **ArgToArray(const char *Args) {
 
 	const char **ArgArray = (const char **)malloc(Count * sizeof(const char *));
 
-	CHECK_ALLOC_RESULT(ArgArray, malloc) {
+	if (AllocFailed(ArgArray)) {
 		return NULL;
-	} CHECK_ALLOC_RESULT_END;
+	}
 
 	for (int i = 0; i < Count; i++) {
 		ArgArray[i] = ArgGet(Args, i + 1);
@@ -198,13 +197,18 @@ const char **ArgDupArray(const char **ArgV) {
 
 	Dup = (char **)malloc(Count * sizeof(char *) + Len + 2);
 
+	if (AllocFailed(Dup)) {
+		return NULL;
+	}
+
 	Offset = (char *)Dup + Count * sizeof(char *) - ArgV[0];
 
 	memcpy(Dup, ArgV, Count * sizeof(char *));
 	memcpy((char *)Dup + Count * sizeof(char *), ArgV[0], Len + 2);
 
-	for (int i = 0; i < Count; i++)
+	for (int i = 0; i < Count; i++) {
 		Dup[i] += Offset;
+	}
 
 	return (const char **)Dup;
 }
@@ -291,13 +295,13 @@ const char **ArgToArray2(const tokendata_t& Tokens) {
 	
 	Pointers = (const char **)malloc(sizeof(const char *) * 33);
 
+	if (AllocFailed(Pointers)) {
+		return NULL;
+	}
+
 	memset(Pointers, 0, sizeof(const char *) * 33);
 
-	CHECK_ALLOC_RESULT(Pointers, malloc) {
-		return NULL;
-	} CHECK_ALLOC_RESULT_END;
-
-	for (unsigned int i = 0; i < min(Tokens.Count, 32); i++) {
+	for (unsigned int i = 0; i < min(Tokens.Count, (unsigned int)32); i++) {
 		Pointers[i] = Tokens.Pointers[i] + Tokens.String;
 	}
 
@@ -342,7 +346,7 @@ unsigned int ArgCount2(const tokendata_t& Tokens) {
  * @param Port the port
  * @param BindIp the ip address/hostname which should be used for binding the socket
  */
-SOCKET SocketAndConnect(const char *Host, unsigned short Port, const char *BindIp) {
+SOCKET SocketAndConnect(const char *Host, unsigned int Port, const char *BindIp) {
 	unsigned long lTrue = 1;
 	sockaddr_in sin, sloc;
 	SOCKET Socket;
@@ -354,15 +358,17 @@ SOCKET SocketAndConnect(const char *Host, unsigned short Port, const char *BindI
 		return INVALID_SOCKET;
 	}
 
-	Socket = safe_socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+	Socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
 	if (Socket == INVALID_SOCKET) {
 		return INVALID_SOCKET;
 	}
 
-	safe_ioctlsocket(Socket, FIONBIO, &lTrue);
+	ioctlsocket(Socket, FIONBIO, &lTrue);
 
 	if (BindIp && *BindIp) {
+		memset(&sloc, 0, sizeof(sloc));
+
 		sloc.sin_family = AF_INET;
 		sloc.sin_port = 0;
 
@@ -378,9 +384,10 @@ SOCKET SocketAndConnect(const char *Host, unsigned short Port, const char *BindI
 			sloc.sin_addr.s_addr = addr;
 		}
 
-		safe_bind(Socket, (sockaddr *)&sloc, sizeof(sloc));
+		bind(Socket, (sockaddr *)&sloc, sizeof(sloc));
 	}
 
+	memset(&sin, 0, sizeof(sin));
 	sin.sin_family = AF_INET;
 	sin.sin_port = htons(Port);
 
@@ -396,14 +403,14 @@ SOCKET SocketAndConnect(const char *Host, unsigned short Port, const char *BindI
 		sin.sin_addr.s_addr = addr;
 	}
 
-	code = safe_connect(Socket, (const sockaddr *)&sin, sizeof(sin));
+	code = connect(Socket, (const sockaddr *)&sin, sizeof(sin));
 
 #ifdef _WIN32
-	if (code != 0 && safe_errno() != WSAEWOULDBLOCK) {
+	if (code != 0 && errno != WSAEWOULDBLOCK) {
 #else
-	if (code != 0 && safe_errno() != EINPROGRESS) {
+	if (code != 0 && errno != EINPROGRESS) {
 #endif
-		safe_closesocket(Socket);
+		closesocket(Socket);
 
 		return INVALID_SOCKET;
 	}
@@ -419,44 +426,60 @@ SOCKET SocketAndConnect(const char *Host, unsigned short Port, const char *BindI
  * @param Host the host's ip address
  * @param BindIp the ip address which should be used for binding the socket
  */
-SOCKET SocketAndConnectResolved(const sockaddr *Host, const sockaddr *BindIp) {
+SOCKET SocketAndConnectResolved(const sockaddr *Host, const sockaddr *BindIp, int *error) {
 	unsigned long lTrue = 1;
 	int Code, Size;
 
-	SOCKET Socket = safe_socket(Host->sa_family, SOCK_STREAM, IPPROTO_TCP);
+	SOCKET Socket = socket(Host->sa_family, SOCK_STREAM, IPPROTO_TCP);
+
+#ifdef _WIN32
+	*error = WSAGetLastError();
+#else
+	*error = errno;
+#endif
 
 	if (Socket == INVALID_SOCKET) {
 		return INVALID_SOCKET;
 	}
 
-	safe_ioctlsocket(Socket, FIONBIO, &lTrue);
+	ioctlsocket(Socket, FIONBIO, &lTrue);
 
 	if (BindIp != NULL) {
-		safe_bind(Socket, (sockaddr *)BindIp, SOCKADDR_LEN(BindIp->sa_family));
-	}
-
-#ifdef IPV6
-	if (Host->sa_family == AF_INET) {
-#endif
-		Size = sizeof(sockaddr_in);
-#ifdef IPV6
-	} else {
-		Size = sizeof(sockaddr_in6);
-	}
+		if (bind(Socket, (sockaddr *)BindIp, SOCKADDR_LEN(BindIp->sa_family)) != 0) {
+#ifdef _WIN32
+			*error = WSAGetLastError();
+#else
+			*error = errno;
 #endif
 
-	Code = safe_connect(Socket, Host, Size);
+			closesocket(Socket);
+
+			return INVALID_SOCKET;
+		}
+	}
+
+	Size = SOCKADDR_LEN(Host->sa_family);
+
+	Code = connect(Socket, Host, Size);
 
 #ifdef _WIN32
-	if (Code != 0 && safe_errno() != WSAEWOULDBLOCK) {
+	*error = WSAGetLastError();
 #else
-	if (Code != 0 && safe_errno() != EINPROGRESS) {
+	*error = errno;
 #endif
-		safe_closesocket(Socket);
+
+#ifdef _WIN32
+	if (Code != 0 && *error != WSAEWOULDBLOCK) {
+#else
+	if (Code != 0 && *error != EINPROGRESS) {
+#endif
+		closesocket(Socket);
 
 		return INVALID_SOCKET;
 	}
 
+	*error = 0;
+
 	return Socket;
 }
 
@@ -472,7 +495,7 @@ char *NickFromHostmask(const char *Hostmask) {
 	char *Copy;
 	const char *ExclamationMark;
 
-	ExclamationMark = strstr(Hostmask, "!");
+	ExclamationMark = strchr(Hostmask, '!');
 
 	if (ExclamationMark == NULL) {
 		return NULL;
@@ -480,9 +503,7 @@ char *NickFromHostmask(const char *Hostmask) {
 
 	Copy = strdup(Hostmask);
 
-	if (Copy == NULL) {
-		LOGERROR("strdup() failed. Could not parse hostmask (%s).", Hostmask);
-
+	if (AllocFailed(Copy)) {
 		return NULL;
 	}
 
@@ -500,36 +521,36 @@ char *NickFromHostmask(const char *Hostmask) {
  * @param BindIp the IP address this socket should be bound to
  * @param Family address family (i.e. IPv4 or IPv6)
  */
-SOCKET CreateListener(unsigned short Port, const char *BindIp, int Family) {
+SOCKET CreateListener(unsigned int Port, const char *BindIp, int Family) {
 	sockaddr *saddr;
 	sockaddr_in sin;
-#ifdef IPV6
+#ifdef HAVE_IPV6
 	sockaddr_in6 sin6;
-#endif
+#endif /* HAVE_IPV6 */
 	const int optTrue = 1;
 	bool Bound = false;
 	SOCKET Listener;
 	hostent *hent;
 
-	Listener = safe_socket(Family, SOCK_STREAM, IPPROTO_TCP);
+	Listener = socket(Family, SOCK_STREAM, IPPROTO_TCP);
 
 	if (Listener == INVALID_SOCKET) {
 		return INVALID_SOCKET;
 	}
 
 #ifndef _WIN32
-	safe_setsockopt(Listener, SOL_SOCKET, SO_REUSEADDR, (char *)&optTrue, sizeof(optTrue));
+	setsockopt(Listener, SOL_SOCKET, SO_REUSEADDR, (char *)&optTrue, sizeof(optTrue));
 #endif
 
-#ifdef IPV6
+#ifdef HAVE_IPV6
 	if (Family == AF_INET) {
-#endif
+#endif /* HAVE_IPV6 */
 		sin.sin_family = AF_INET;
 		sin.sin_port = htons(Port);
 
 		saddr = (sockaddr *)&sin;
-#ifdef IPV6
-	} else {
+#ifdef HAVE_IPV6 /* HAVE_IPV6 */
+	} else if (Family == AF_INET6) {
 		memset(&sin6, 0, sizeof(sin6));
 		sin6.sin6_family = AF_INET6;
 		sin6.sin6_port = htons(Port);
@@ -537,49 +558,55 @@ SOCKET CreateListener(unsigned short Port, const char *BindIp, int Family) {
 		saddr = (sockaddr *)&sin6;
 
 #if !defined(_WIN32) && defined(IPV6_V6ONLY)
-		safe_setsockopt(Listener, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&optTrue, sizeof(optTrue));
+		setsockopt(Listener, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&optTrue, sizeof(optTrue));
 #endif
+	} else {
+		closesocket(Listener);
+
+		return INVALID_SOCKET;
 	}
-#endif
+#endif /* HAVE_IPV6 */
 
 	if (BindIp) {
 		hent = gethostbyname(BindIp);
 
 		if (hent) {
-#ifdef IPV6
-			if (Family = AF_INET) {
-#endif
+#ifdef HAVE_IPV6
+			if (Family == AF_INET) {
+#endif /* HAVE_IPV6 */
 				sin.sin_addr.s_addr = ((in_addr*)hent->h_addr_list[0])->s_addr;
-#ifdef IPV6
+#ifdef HAVE_IPV6
 			} else {
 				memcpy(&(sin6.sin6_addr.s6_addr), &(((in6_addr*)hent->h_addr_list[0])->s6_addr), sizeof(in6_addr));
 			}
-#endif
+#endif /* HAVE_IPV6 */
 
 			Bound = true;
 		}
 	}
 
 	if (!Bound) {
-#ifdef IPV6
+#ifdef HAVE_IPV6
 		if (Family == AF_INET) {
-#endif
+#endif /* HAVE_IPV6 */
 			sin.sin_addr.s_addr = htonl(INADDR_ANY);
-#ifdef IPV6
+#ifdef HAVE_IPV6
 		} else {
-			memcpy(&(sin6.sin6_addr.s6_addr), &in6addr_any, sizeof(in6_addr));
+			const struct in6_addr v6any = IN6ADDR_ANY_INIT;
+
+			memcpy(&(sin6.sin6_addr.s6_addr), &v6any, sizeof(in6_addr));
 		}
-#endif
+#endif /* HAVE_IPV6 */
 	}
 
-	if (safe_bind(Listener, saddr, SOCKADDR_LEN(Family)) != 0) {
-		safe_closesocket(Listener);
+	if (bind(Listener, saddr, SOCKADDR_LEN(Family)) != 0) {
+		closesocket(Listener);
 
 		return INVALID_SOCKET;
 	}
 
-	if (safe_listen(Listener, SOMAXCONN) != 0) {
-		safe_closesocket(Listener);
+	if (listen(Listener, SOMAXCONN) != 0) {
+		closesocket(Listener);
 
 		return INVALID_SOCKET;
 	}
@@ -594,38 +621,71 @@ SOCKET CreateListener(unsigned short Port, const char *BindIp, int Family) {
  * a string representation of the hash.
  *
  * @param String the string which should be hashed
+ * @param Salt the salt value
+ * @param BrokenAlgo whether to use the broken algorithm
  */
-const char *UtilMd5(const char *String, const char *Salt) {
+const char *UtilMd5(const char *String, const char *Salt, bool BrokenAlgo) {
+#ifdef HAVE_LIBSSL
+	MD5_CTX context;
+#else /* HAVE_LIBSSL */
 	sMD5_CTX context;
+#endif /* HAVE_LIBSSL */
+	broken_sMD5_CTX broken_context;
 	unsigned char digest[16];
 	char *StringAndSalt, *StringPtr;
 	static char *SaltAndResult = NULL;
+	int rc;
 
 	free(SaltAndResult);
 
 	if (Salt != NULL) {
-		asprintf(&StringAndSalt, "%s%s", String, Salt);
+		rc = asprintf(&StringAndSalt, "%s%s", String, Salt);
 	} else {
-		asprintf(&StringAndSalt, "%s", String);
+		rc = asprintf(&StringAndSalt, "%s", String);
 	}
 
-	MD5Init(&context);
-	MD5Update(&context, (unsigned char *)StringAndSalt, strlen(StringAndSalt));
-	MD5Final(digest, &context);
+	if (RcFailed(rc)) {
+		g_Bouncer->Fatal();
+	}
+
+	if (!BrokenAlgo) {
+#ifdef HAVE_LIBSSL
+		MD5_Init(&context);
+		MD5_Update(&context, (unsigned char *)StringAndSalt, strlen(StringAndSalt));
+		MD5_Final(digest, &context);
+#else /* HAVE_LIBSSL */
+		MD5Init(&context);
+		MD5Update(&context, (unsigned char *)StringAndSalt, strlen(StringAndSalt));
+		MD5Final(digest, &context);
+#endif /* HAVE_LIBSSL */
+	} else {
+		broken_MD5Init(&broken_context);
+		broken_MD5Update(&broken_context, (unsigned char *)StringAndSalt, strlen(StringAndSalt));
+		broken_MD5Final(digest, &broken_context);
+	}
 
 	free(StringAndSalt);
 
 	if (Salt != NULL) {
 		SaltAndResult = (char *)malloc(strlen(Salt) + 50);
+
+		if (AllocFailed(SaltAndResult)) {
+			g_Bouncer->Fatal();
+		}
+
 		strmcpy(SaltAndResult, Salt, strlen(Salt) + 50);
 		strmcat(SaltAndResult, "$", strlen(Salt) + 50);
 		StringPtr = SaltAndResult + strlen(SaltAndResult);
 	} else {
 		StringPtr = SaltAndResult = (char *)malloc(50);
+
+		if (AllocFailed(SaltAndResult)) {
+			g_Bouncer->Fatal();
+		}
 	}
 
 	for (int i = 0; i < 16; i++) {
-#undef sprintf
+		/* TODO: don't use sprintf */
 		sprintf(StringPtr + i * 2, "%02x", digest[i]);
 	}
 
@@ -642,7 +702,7 @@ const char *GenerateSalt(void) {
 
 	for (unsigned int i = 0; i < sizeof(Salt) - 1; i++) {
 		do {
-			Salt[i] = 0x21 + rand() % (0xFF - 0x21);
+			Salt[i] = '0' + rand() % ('^' - '0');
 		} while (Salt[i] == '$');
 	}
 
@@ -671,6 +731,11 @@ const char *SaltFromHash(const char *Hash) {
 	free(Salt);
 
 	Salt = (char *)malloc(HashSign - Hash + 1);
+
+	if (AllocFailed(Salt)) {
+		g_Bouncer->Fatal();
+	}
+
 	strmcpy(Salt, Hash, HashSign - Hash + 1);
 
 	return Salt;
@@ -702,7 +767,7 @@ static void DestroyCommandT(command_t *Command) {
 	free(Command->Category);
 	free(Command->Description);
 	free(Command->HelpText);
-	free(Command);
+	delete Command;
 }
 
 /**
@@ -724,22 +789,18 @@ void AddCommand(commandlist_t *Commands, const char *Name, const char *Category,
 	}
 
 	if (*Commands == NULL) {
-		*Commands = new CHashtable<command_t *, false, 16>();
-
-		if (*Commands == NULL) {
-			LOGERROR("new operator failed. Could not add command.");
+		*Commands = new CHashtable<command_t *, false>();
 
+		if (AllocFailed(*Commands)) {
 			return;
 		}
 
 		(*Commands)->RegisterValueDestructor(DestroyCommandT);
 	}
 
-	Command = (command_t *)malloc(sizeof(command_t));
-
-	if (Command == NULL) {
-		LOGERROR("malloc() failed. Could not add command.");
+	Command = new command_t;
 
+	if (AllocFailed(Command)) {
 		return;
 	}
 
@@ -807,24 +868,11 @@ const char *IpToString(sockaddr *Address) {
 	static char Buffer[256];
 
 #ifdef _WIN32
-	sockaddr *Copy = (sockaddr *)malloc(SOCKADDR_LEN(Address->sa_family));
 	DWORD BufferLength = sizeof(Buffer);
 
-	CHECK_ALLOC_RESULT(Copy, malloc) {
-		return "<out of memory>";
-	} CHECK_ALLOC_RESULT_END;
-
-	memcpy(Copy, Address, SOCKADDR_LEN(Address->sa_family));
-
-	if (Address->sa_family == AF_INET) {
-		((sockaddr_in *)Address)->sin_port = 0;
-	} else {
-		((sockaddr_in6 *)Address)->sin6_port = 0;
+	if (WSAAddressToString(Address, SOCKADDR_LEN(Address->sa_family), NULL, Buffer, &BufferLength) != 0) {
+		return NULL;
 	}
-
-	WSAAddressToString(Address, SOCKADDR_LEN(Address->sa_family), NULL, Buffer, &BufferLength);
-
-	free(Copy);
 #else
 	void *IpAddress;
 
@@ -834,7 +882,9 @@ const char *IpToString(sockaddr *Address) {
 		IpAddress = &(((sockaddr_in6 *)Address)->sin6_addr);
 	}
 
-	inet_ntop(Address->sa_family, IpAddress, Buffer, sizeof(Buffer));
+	if (inet_ntop(Address->sa_family, IpAddress, Buffer, sizeof(Buffer)) == NULL) {
+		return NULL;
+	}
 #endif
 
 	return Buffer;
@@ -866,7 +916,8 @@ int CompareAddress(const sockaddr *pA, const sockaddr *pB) {
 	}
 
 	if (pA->sa_family == AF_INET6) {
-		if (((sockaddr_in6 *)pA)->sin6_addr.s6_addr == ((sockaddr_in6 *)pB)->sin6_addr.s6_addr) {
+		if (memcmp(&(((sockaddr_in6 *)pA)->sin6_addr.s6_addr),
+				&(((sockaddr_in6 *)pB)->sin6_addr.s6_addr), sizeof(in6_addr)) == 0) {
 			return 0;
 		} else {
 			return 1;
@@ -879,17 +930,17 @@ int CompareAddress(const sockaddr *pA, const sockaddr *pB) {
 /**
  * StrTrim
  *
- * Removes leading and trailing spaces from a string.
+ * Removes the specified char from the start and end of the string.
  *
  * @param String the string
  */
-void StrTrim(char *String) {
+void StrTrim(char *String, char Character) {
 	size_t Length = strlen(String);
 	size_t Offset = 0, i;
 
-	// remove leading spaces
+	// remove leading chars
 	for (i = 0; i < Length; i++) {
-		if (String[i] == ' ') {
+		if (String[i] == Character) {
 			Offset++;
 		} else {
 			break;
@@ -902,8 +953,8 @@ void StrTrim(char *String) {
 		}
 	}
 
-	// remove trailing spaces
-	while (String[strlen(String) - 1] == ' ') {
+	// remove trailing chars
+	while (String[strlen(String) - 1] == Character) {
 		String[strlen(String) - 1] = '\0';
 	}
 }
@@ -924,27 +975,6 @@ int SetPermissions(const char *Filename, int Modes) {
 #endif
 }
 
-/**
- * RegisterZone
- *
- * Registers a zone information object.
- *
- * @param ZoneInformation the zone information object
- */
-bool RegisterZone(CZoneInformation *ZoneInformation) {
-	if (g_Bouncer != NULL) {
-		g_Bouncer->RegisterZone(ZoneInformation);
-
-		return true;
-	} else {
-#ifdef _DEBUG
-		safe_printf("Error in RegisterZone!\n");
-#endif
-
-		return false;
-	}
-}
-
 /**
  * FreeString
  *
@@ -956,17 +986,6 @@ void FreeString(char *String) {
 	free(String);
 }
 
-/**
- * FreeUString
- *
- * Calls ufree() on a string.
- *
- * @param String the string
- */
-void FreeUString(char *String) {
-	ufree(String);
-}
-
 /**
  * strmcpy
  *
@@ -1012,372 +1031,34 @@ char *strmcat(char *Destination, const char *Source, size_t Size) {
 	return Destination;
 }
 
-#if defined(_DEBUG) && defined(_WIN32)
-LONG WINAPI GuardPageHandler(EXCEPTION_POINTERS *Exception) {
-	char charSymbol[sizeof(SYMBOL_INFO) + 200];
-	SYMBOL_INFO *Symbol = (SYMBOL_INFO *)charSymbol;
-	IMAGEHLP_LINE64 Line;
-
-	if (Exception->ExceptionRecord->ExceptionCode == STATUS_GUARD_PAGE_VIOLATION) {
-		Symbol->SizeOfStruct = sizeof(SYMBOL_INFO);
-		Symbol->MaxNameLen = sizeof(charSymbol) - sizeof(SYMBOL_INFO);
-		SymFromAddr(GetCurrentProcess(), (DWORD64)Exception->ExceptionRecord->ExceptionAddress, NULL, Symbol);
-
-		Line.SizeOfStruct = sizeof(Line);
-
-		if (SymGetLineFromAddr64(GetCurrentProcess(), (DWORD64)Exception->ExceptionRecord->ExceptionAddress, 0, &Line)) {
-			safe_printf("Hit guard page at %s. (%s:%d)\n", Symbol->Name, Line.FileName, Line.LineNumber);
-		} else {
-			safe_printf("Hit guard page at %s.\n", Symbol->Name);
-		}
-
-		return EXCEPTION_CONTINUE_EXECUTION;
-	}
-
-	return EXCEPTION_CONTINUE_SEARCH;
-}
-
-void mstacktrace(void) {
-	STACKFRAME64 Frame;
-	DWORD FramePointer;
-
-//	return; /* ... */
-
-	__asm mov FramePointer, ebp
-
-	memset(&Frame, 0, sizeof(Frame));
-
-	Frame.AddrPC.Offset = (DWORD64)_ReturnAddress();
-	Frame.AddrPC.Mode = AddrModeFlat;
-	Frame.AddrFrame.Offset = FramePointer;
-	Frame.AddrFrame.Mode = AddrModeFlat;
-
-	if (g_Bouncer != NULL) {
-		g_Bouncer->Log("---");
-	}
-
-	while (StackWalk64(IMAGE_FILE_MACHINE_I386, GetCurrentProcess(), GetCurrentThread(), &Frame, NULL, NULL, SymFunctionTableAccess64, SymGetModuleBase64, NULL)) {
-		char charSymbol[sizeof(SYMBOL_INFO) + 200];
-		SYMBOL_INFO *Symbol = (SYMBOL_INFO *)charSymbol;
-
-		if (Frame.AddrPC.Offset == 0) {
-			break;
-		}
-
-		Symbol->SizeOfStruct = sizeof(SYMBOL_INFO);
-		Symbol->MaxNameLen = sizeof(charSymbol) - sizeof(SYMBOL_INFO);
-		SymFromAddr(GetCurrentProcess(), (DWORD64)Frame.AddrPC.Offset, NULL, Symbol);
-
-		if (g_Bouncer != NULL) {
-			g_Bouncer->Log("Call from %s", Symbol->Name);
-		}
-	}
-}
-
-void mmark(void *Block) {
-	DWORD Dummy;
-	mblock *RealBlock;
-
-	if (Block == NULL) {
-		return;
-	}
-
-	RealBlock = (mblock *)Block - 1;
-
-	VirtualProtect(RealBlock, sizeof(mblock), PAGE_READWRITE, &Dummy);
-
-	if (RealBlock->Marker != BLOCKMARKER) {
-		return;
-	}
-
-	VirtualProtect(RealBlock, sizeof(mblock) + RealBlock->Size, PAGE_READWRITE, &Dummy);
-}
-#endif
-
-void mclaimmanager(mmanager_t *Manager) {
-	Manager->ReferenceCount++;
-}
-
-void mreleasemanager(mmanager_t *Manager) {
-	if (Manager != NULL) {
-		Manager->ReferenceCount--;
-
-		if (Manager->ReferenceCount == 0) {
-			free(Manager);
-		}
-	}
-}
-
-void *mmalloc(size_t Size, CUser *Owner) {
-#if defined(_DEBUG) && defined(_WIN32)
-	DWORD Dummy;
-#endif
-	mblock *Block;
-
-	if (Owner != NULL && !Owner->MemoryAddBytes(Size)) {
-		return NULL;
-	}
-
-#if defined(_DEBUG) && defined(_WIN32)
-	Block = (mblock *)VirtualAlloc(NULL, sizeof(mblock) + Size, MEM_RESERVE |  MEM_COMMIT, PAGE_READWRITE);
-#else
-	Block = (mblock *)malloc(sizeof(mblock) + Size);
-#endif
-
-	if (Block == NULL) {
-		if (Owner != NULL) {
-			Owner->MemoryRemoveBytes(Size);
-		}
-
-		return NULL;
-	}
-
-	Block->Size = Size;
-
-	if (Owner != NULL) {
-		Block->Manager = Owner->MemoryGetManager();
-	 	mclaimmanager(Block->Manager);
-	} else {
-		Block->Manager = NULL;
-	}
-
-#if defined(_DEBUG) && defined(_WIN32)
-	Block->Marker = BLOCKMARKER;
-#endif
-
-#if defined(_DEBUG) && defined(_WIN32)
-/*	if (Block->Manager != NULL && g_Bouncer != NULL) {
-		safe_printf("%p = mmalloc(%d, %p), mgr refcount = %d\n", Block + 1, Size, Owner, Owner->MemoryGetManager()->ReferenceCount);
-	}*/
-
-	VirtualProtect(Block, sizeof(mblock) + Size, PAGE_READWRITE | PAGE_GUARD, &Dummy);
-
-//	mstacktrace();
-#endif
-
-	return Block + 1;
-}
-
-void mfree(void *Block) {
-	mblock *RealBlock;
-	unsigned int DebugRefCount;
-
-	if (Block == NULL) {
-		return;
-	}
-
-	mmark(Block);
-
-	RealBlock = (mblock *)Block - 1;
-
-#if defined(_DEBUG) && defined(_WIN32)
-	if (RealBlock->Marker == 0xaaaaaaaa) {
-		DebugBreak();
-	}
-
-	if (RealBlock->Marker != BLOCKMARKER) {
-		free(Block);
-
-		return;
-	}
-#endif
-
-#ifdef _DEBUG
-	RealBlock->Marker = 0xaaaaaaaa;
-#endif
-
-	if (RealBlock->Manager != NULL) {
-		DebugRefCount = RealBlock->Manager->ReferenceCount - 1;
-	}
-
-	if (RealBlock->Manager != NULL && RealBlock->Manager->RealManager != NULL) {
-		RealBlock->Manager->RealManager->MemoryRemoveBytes(RealBlock->Size);
-
-		mreleasemanager(RealBlock->Manager);
-	}
-
-#if defined(_DEBUG) && defined(_WIN32)
-/*	if (RealBlock->Manager != NULL && g_Bouncer != NULL) {
-		safe_printf("mfree(%p), mgr refcount = %d\n", Block, DebugRefCount);
-	}*/
-
-	VirtualFree(RealBlock, 0, MEM_RELEASE);
-
-//	mstacktrace();
-#else
-	free(RealBlock);
-#endif
-}
-
-void *mrealloc(void *Block, size_t NewSize, CUser *Manager) {
-#if defined(_DEBUG) && defined(_WIN32)
-	DWORD Dummy;
-#endif
-	mblock *RealBlock, *NewRealBlock;
-	mmanager_t *NewManager;
-
-	if (Block == NULL) {
-		return mmalloc(NewSize, Manager);
-	}
-
-	RealBlock = (mblock *)Block - 1;
-
-	mmark(Block);
-
-	if (RealBlock->Manager->RealManager != NULL) {
-		RealBlock->Manager->RealManager->MemoryRemoveBytes(RealBlock->Size);
-	}
-
-	if (Manager != NULL && !Manager->MemoryAddBytes(NewSize)) {
-		return NULL;
-	}
-
-#if defined(_DEBUG) && defined(_WIN32)
-	NewRealBlock = (mblock *)VirtualAlloc(NULL, sizeof(mblock) + NewSize, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
-
-	if (NewRealBlock != NULL) {
-		memcpy(NewRealBlock, RealBlock, sizeof(mblock) + RealBlock->Size);
-		mfree(Block);
-	}
-#else
-	NewRealBlock = (mblock *)realloc(RealBlock, sizeof(mblock) + NewSize);
-#endif
-
-	if (NewRealBlock == NULL) {
-		if (Manager != NULL) {
-			Manager->MemoryRemoveBytes(RealBlock->Size);
-		}
-
-		return NULL;
-	}
-
-	NewRealBlock->Size = NewSize;
-
-	NewManager = Manager->MemoryGetManager();
-	mclaimmanager(NewManager);
-
-	mreleasemanager(NewRealBlock->Manager);
-	NewRealBlock->Manager = NewManager;
-
-#if defined(_DEBUG) && defined(_WIN32)
-	NewRealBlock->Marker = BLOCKMARKER;
-
-	VirtualProtect(NewRealBlock, sizeof(mblock) + NewSize, PAGE_READWRITE | PAGE_GUARD, &Dummy);
-
-/*	if (NewManager != NULL && g_Bouncer != NULL) {
-		safe_printf("%p = mrealloc(%p, %d, %p), mgr refcount = %d\n", NewRealBlock + 1, Block, NewSize, Manager, NewManager->ReferenceCount);
-	}*/
-//	mstacktrace();
-#endif
-
-	return NewRealBlock + 1;
-}
-
-char *mstrdup(const char *String, CUser *Manager) {
-	size_t Length;
-	char *Copy;
-
-	Length = strlen(String);
-
-	Copy = (char *)mmalloc(Length + 1, Manager);
-
-	if (Copy == NULL) {
-		return NULL;
-	}
-
-	mmark(Copy);
-	memcpy(Copy, String, Length + 1);
-
-	return Copy;
-}
-
-extern "C" struct pollfd *registersocket(int Socket) {
-	pollfd *PollFd = NULL;
-	pollfd NewPollFd;
-	bool NewStruct = true;
-	CVector<pollfd> *PollFds;
-
-	unregistersocket(Socket);
-
-	PollFds = &(g_Bouncer->m_PollFds);
-
-	for (unsigned int i = 0; i < PollFds->GetLength(); i++) {
-		if (PollFds->Get(i).fd == INVALID_SOCKET) {
-			PollFd = PollFds->GetAddressOf(i);
-			NewStruct = false;
-
-			break;
-		}
-	}
-
-	if (NewStruct) {
-		PollFd = &NewPollFd;
-	}
-
-	PollFd->fd = Socket;
-	PollFd->events = 0;
-	PollFd->revents = 0;
-
-	if (NewStruct) {
-		if (!PollFds->Insert(*PollFd)) {
-			return NULL;
-		}
-
-		PollFd = PollFds->GetAddressOf(PollFds->GetLength() - 1);
-	}
-
-	return PollFd;
-}
-
-extern "C" void unregistersocket(int Socket) {
-	CVector<pollfd> *PollFds;
-
-	PollFds = &(g_Bouncer->m_PollFds);
-
-	for (unsigned int i = 0; i < PollFds->GetLength(); i++) {
-		pollfd *PFd = PollFds->GetAddressOf(i);
-
-		if (PFd->fd == Socket) {
-			PFd->fd = INVALID_SOCKET;
-			PFd->events = 0;
-		}
-	}
-}
-
 const sockaddr *HostEntToSockAddr(hostent *HostEnt) {
-	static char Buffer[MAX_SOCKADDR_LEN];
-	sockaddr_in *sin4;
-	sockaddr_in6 *sin6;
+	static sockaddr_storage sin;
 
-	memset(Buffer, 0, sizeof(Buffer));
+	memset(&sin, 0, sizeof(sin));
+
+	sin.ss_family = HostEnt->h_addrtype;
 
-#ifdef IPV6
 	if (HostEnt->h_addrtype == AF_INET) {
-#endif
-		sin4 = (sockaddr_in *)Buffer;
-
-		sin4->sin_family = AF_INET;
-		sin4->sin_port = 0;
-		memcpy(&(sin4->sin_addr), HostEnt->h_addr_list[0], sizeof(in_addr));
-#ifdef IPV6
+		((sockaddr_in *)&sin)->sin_port = 0;
+		memcpy(&(((sockaddr_in *)&sin)->sin_addr), HostEnt->h_addr_list[0], sizeof(in_addr));
+#ifdef HAVE_IPV6
+	} else if (HostEnt->h_addrtype == AF_INET6) {
+		((sockaddr_in6 *)&sin)->sin6_port = 0;
+		memcpy(&(((sockaddr_in6 *)&sin)->sin6_addr), HostEnt->h_addr_list[0], sizeof(in6_addr));
+#endif /* HAVE_IPV6 */
 	} else {
-		sin6 = (sockaddr_in6 *)Buffer;
-
-		sin6->sin6_family = AF_INET6;
-		sin6->sin6_port = 0;
-		memcpy(&(sin6->sin6_addr), HostEnt->h_addr_list[0], sizeof(in6_addr));
+		return NULL;
 	}
-#endif
 
-	return (sockaddr *)Buffer;
+	return (sockaddr *)&sin;
 }
 
 bool StringToIp(const char *IP, int Family, sockaddr *SockAddr, socklen_t Length) {
-	socklen_t *LengthPtr = &Length;
-
 	memset(SockAddr, 0, Length);
 
 #ifdef _WIN32
+	socklen_t *LengthPtr = &Length;
+
 	if (WSAStringToAddress(const_cast<char *>(IP), Family, NULL, SockAddr, LengthPtr) != 0) {
 		return false;
 	}
@@ -1394,44 +1075,36 @@ bool StringToIp(const char *IP, int Family, sockaddr *SockAddr, socklen_t Length
 	return true;
 }
 
-static int safe_passwd_cb(char *Buffer, int Size, int RWFlag, void *Cookie) {
+static int passwd_cb(char *Buffer, int Size, int RWFlag, void *Cookie) {
 	int Result;
 	char ConfirmBuffer[128];
 
-	if (g_Bouncer->IsDaemonized()) {
-		LOGERROR("Password is required to decrypt the SSL certificate. However shroudBNC"
-			" is daemonized and cannot read user input. Re-run shroudBNC with --foreground please.");
-		g_Bouncer->Fatal();
-
-		return -1;
-	}
-
 	if (Size > 128) {
 		Size = 128; // nobody could seriously have such a passphrase...
 	}
 
 	do {
-		safe_print("PEM passphrase: ");
+		printf("PEM passphrase: ");
 		
-		Result = safe_scan_passwd(Buffer, Size);
-		safe_print("\n");
+		Result = sn_getline_passwd(Buffer, Size);
+		printf("\n");
 		
 		if (Result <= 0) {
 			return 0;
 		}
 
 		if (RWFlag == 1) {
-			safe_print("Confirm PEM passphrase: ");
+			printf("Confirm PEM passphrase: ");
 			
-			Result = safe_scan_passwd(ConfirmBuffer, sizeof(ConfirmBuffer));
-			safe_print("\n");
+			Result = sn_getline_passwd(ConfirmBuffer, sizeof(ConfirmBuffer));
+			printf("\n");
 
 			if (Result <= 0) {
 				return 0;
 			}
 
 			if (strcmp(Buffer, ConfirmBuffer) != 0) {
-				safe_print("The passwords you specified do not match. Please try again.\n");
+				printf("The passwords you specified do not match. Please try again.\n");
 
 				continue;
 			}
@@ -1443,8 +1116,216 @@ static int safe_passwd_cb(char *Buffer, int Size, int RWFlag, void *Cookie) {
 	return strlen(Buffer);
 }
 
-void SSL_CTX_set_safe_passwd_cb(SSL_CTX *Context) {
-#ifdef USESSL
-	SSL_CTX_set_default_passwd_cb(Context, safe_passwd_cb);
+void SSL_CTX_set_passwd_cb(SSL_CTX *Context) {
+#ifdef HAVE_LIBSSL
+	SSL_CTX_set_default_passwd_cb(Context, passwd_cb);
+#endif /* HAVE_LIBSSL */
+}
+
+#ifndef HAVE_POLL
+/*
+ *  prt
+ *
+ *  Copyright 1994 University of Washington
+ *
+ *  Permission is hereby granted to copy this software, and to
+ *  use and redistribute it, except that this notice may not be
+ *  removed.  The University of Washington does not guarantee
+ *  that this software is suitable for any purpose and will not
+ *  be held liable for any damage it may cause.
+ */
+
+/*
+**  emulate poll() for those platforms (Ultrix) that don't have it.
+*/
+int poll(pollfd *fds, unsigned long nfds, int timo) {
+	struct timeval timeout, *toptr;
+	fd_set ifds, ofds, efds, *ip, *op;
+	int i, rc, n;
+
+	FD_ZERO(&ifds);
+	FD_ZERO(&ofds);
+	FD_ZERO(&efds);
+
+	for (i = 0, n = -1, op = ip = 0; i < (int)nfds; ++i) {
+		fds[i].revents = 0;
+
+		if (fds[i].fd < 0) {
+			continue;
+		}
+
+		if (fds[i].fd > n) {
+			n = fds[i].fd;
+		}
+
+		if (fds[i].events & (POLLIN|POLLPRI)) {
+			ip = &ifds;
+			FD_SET(fds[i].fd, ip);
+		}
+
+		if (fds[i].events & POLLOUT) {
+			op = &ofds;
+			FD_SET(fds[i].fd, op);
+		}
+
+		FD_SET(fds[i].fd, &efds);
+	}
+
+	if (timo < 0) {
+		toptr = 0;
+	} else {
+		toptr = &timeout;
+		timeout.tv_sec = timo / 1000;
+		timeout.tv_usec = (timo - timeout.tv_sec * 1000) * 1000;
+	}
+
+	rc = select(++n, ip, op, &efds, toptr);
+
+	if (rc <= 0) {
+		return rc;
+	}
+
+	for (i = 0, n = 0; i < (int)nfds; ++i) {
+		if (fds[i].fd < 0) {
+			continue;
+		}
+
+		if (fds[i].events & (POLLIN|POLLPRI) && FD_ISSET(fds[i].fd, &ifds)) {
+			fds[i].revents |= POLLIN;
+		}
+
+		if (fds[i].events & POLLOUT && FD_ISSET(fds[i].fd, &ofds)) {
+			fds[i].revents |= POLLOUT;
+		}
+
+		if (FD_ISSET(fds[i].fd, &efds)) {
+			/* Some error was detected ... should be some way to know. */
+			fds[i].revents |= POLLHUP;
+		}
+	}
+
+	return rc;
+}
+#endif /* !HAVE_POLL */
+
+int sn_getline(char *buf, size_t size) {
+	if (fgets(buf, size, stdin) == NULL) {
+		return -1;
+	}
+
+
+	for (char *p = buf + strlen(buf); p >= buf; p--) {
+		if (*p == '\r' || *p == '\n') {
+			*p = '\0';
+
+			break;
+		}
+	}
+
+	return 1;
+}
+
+int sn_getline_passwd(char *buf, size_t size) {
+	bool term_succeeded;
+	int result;
+#ifndef _WIN32
+	termios term_old, term_new;
+#else
+	HANDLE StdInHandle;
+	DWORD ConsoleModes, NewConsoleModes;
 #endif
+
+#ifndef _WIN32
+	if (tcgetattr(STDIN_FILENO, &term_old) == 0) {
+		memcpy(&term_new, &term_old, sizeof(term_old));
+		term_new.c_lflag &= ~ECHO;
+
+		tcsetattr(STDIN_FILENO, TCSANOW, &term_new);
+
+		term_succeeded = true;
+	} else {
+		term_succeeded = false;
+	}
+#else
+	StdInHandle = GetStdHandle(STD_INPUT_HANDLE);
+
+	if (StdInHandle != INVALID_HANDLE_VALUE) {
+		if (GetConsoleMode(StdInHandle, &ConsoleModes)) {
+			NewConsoleModes = ConsoleModes & ~ENABLE_ECHO_INPUT;
+
+			SetConsoleMode(StdInHandle,NewConsoleModes);
+
+			term_succeeded = true;
+		} else {
+			term_succeeded = false;
+		}
+	} else {
+		term_succeeded = false;
+	}
+#endif
+
+	result = sn_getline(buf, size);
+
+	if (term_succeeded) {
+#ifndef _WIN32
+		tcsetattr(STDIN_FILENO, TCSANOW, &term_old);
+#else
+		SetConsoleMode(StdInHandle, ConsoleModes);
+#endif
+	}
+
+	return result;
+}
+
+bool RcFailedInternal(int ReturnCode, const char *File, int Line) {
+	if (ReturnCode >= 0) {
+		return false;
+	}
+
+	if (g_Bouncer != NULL) {
+		g_Bouncer->Log("Function call in %s:%d failed: %s\n", File, Line, strerror(errno));
+	} else {
+		printf("Function call in %s:%d failed: %s\n", File, Line, strerror(errno));
+	}
+
+	return true;
+}
+
+bool AllocFailedInternal(const void *Ptr, const char *File, int Line) {
+	if (Ptr != NULL) {
+		return false;
+	}
+
+	if (g_Bouncer != NULL) {
+		g_Bouncer->Log("Allocation in %s:%d failed: %s\n", File, Line, strerror(errno));
+	} else {
+		printf("Allocation in %s:%d failed: %s\n", File, Line, strerror(errno));
+	}
+
+	return true;
+}
+
+#ifndef _WIN32
+lt_dlhandle sbncLoadLibrary(const char *Filename) {
+	lt_dlhandle handle = 0;
+	lt_dladvise advise;
+
+	if (!lt_dladvise_init(&advise) && !lt_dladvise_global(&advise)) {
+		handle = lt_dlopenadvise(Filename, advise);
+	}
+
+	lt_dladvise_destroy(&advise);
+
+	return handle;
+}
+#endif
+
+void gfree(void *ptr) {
+	/**
+	 * Win32 might have separate heaps for sbnc and its modules (depending
+	 * on how we link to the CRT) - so in order to free memory that was allocated
+	 * by sbnc (e.g. using asprintf) we need to export a function
+	 * that calls the appropriate version of free().
+	 */
+	free(ptr);
 }
